.BG
.FN rm.fit
.TL
Fit repeated measurements models
.DN
Fit repeated measurements models
.CS
rm.fit(formula = formula(data), data , family="gaussian", 
cov.struct="I",  random.effects  , time, link="log",
subset, start = NULL, trace = T, contrasts = NULL, 
control = nlminb.control(eval.max=50,x.tol=1e-6,rel.tol=1e-6), ...)
.RA
.AG formula
a formula expression as for regression models
of the form  `response ~ predictors'. See the 
documentation of `formula' for details
.AG data
a data list containing the variables occurring in the formula. List
elements are identified by names.  `NA' values are allowed. If data is
missing, an error is produced.
.OA
.AG family
a string identifying the parametric family. Families supported
are `"gaussian"' (default), `"binary"' and `"poisson"'.
.AG cov.struct
a string identifying the covariance structure to use in the Gaussian
case.  Four possible values can occur: `"I"', independence, `"AR(1)"'
first order autoregressive structure, `"I+random"', compound symmetry,
`"AR(1)+random"' first order autoregressive structure+compound
symmetry.  The default value is `"I"'. The last two options can be
used jointly with the input parameter `"random.effects"'.
.AG random.effects
a formula identifying the structure of the random effects.  This is a
formula of the type `response ~ random effects'. Used jointly with
`"I+random"' and `"AR(1)+random"' allows to model random effects in
the linear model for each observation.
.AG time
a string that matches the name of the time variable in data.
By default, the program expects a variable named `"time"' to be present
in the  data list, otherwise the name of the variable playing 
the role of time must be declared by assigning time here
.AG counts
a string that matches the name of the count vector in data.  If the
vector is named `counts', then it is automatically used by the
function, otherwise the name of the vector playing
the role of counts must be declared by assigning counts here.
.AG link
a string identifying the link for the Poisson case. Supported
links are `"log"' (default) and `"identity"'.
.AG subset
expression stating which subset of the data rows  should
be used in the plot and fit. This is a logical  statement of the type
`"covariate1 == "a" & covariate2 > x"' which identifies the observations 
to be selected. The default is `"all observations"'.
.AG contrasts
a list of contrasts to be used for some or all of the factors
appearing as variables in the model formula.  The names in the list
should be the names of the corresponding variables, and the elements
should either be contrast-type matrices (matrices with as many rows as
levels of the factor and with columns linearly independent of each
other and of a column of one's), or else they should be functions that
compute such contrast matrices.
.AG start
a vector of initial values for the nuisance parameters of the likelihood.
See the user manual for details on the use of start in the
Gaussian  case. In the binary and Poisson case, one value is required
for start.
.AG control
a list of algorithmic constants for the optimizer `nlminb'; see the
S-PLUS documentation of `nlminb.control' for details on the possible
control options. By default, `rm.fit()' sets the maximum number of
function evaluations (`eval.max') equal to 50, the parameter function
convergence tolerance (`x.tol') and the relative convergence tolerance
(`rel.tol') equal to 1e-6 and uses the nlminb standard default values
for the remaining options; any setting of control modifies the default
values of the options included in the list and causes the options that
are not set in the list to be replaced by their standard default
values within `nlminb'.
.AG trace
logical flag: if `TRUE',  details of the nonlinear optimization are printed.
By default the flag is set to `TRUE'.
.RT
an object of class `"m"' is returned. See `rm.object' for details.
.DT
.SH FORMULA
The `formula' argument is passed around unevaluated; that is, the
variables mentioned in the formula will be defined when the model
matrices are computed, not when `rm.fit' is initially called.  In
particular, if data is given, all these names should generally be
defined as variables in that data list.
.SH DATA
Data are contained in a list. Elements of the data list can be
matrices or vectors.  At the very least, data must contain a matrix
representing the response variable and a vector or matrix representing
the observation times. Other elements of the list may contain
covariates, possibly time depedent. Rows of matrices are associated
with individuals, while column are associated with the observation
time.
.PP
Each element of the data argument must be identifiable by a
name. Names are assigned to the elements of a list with a standard
assignment of the type:
.sp
	`data <- list(response = matrix(...), time = c(...)}'
.sp
Elements in the data list can appear in any order, the name of the
element being the selecting argument for evaluation.  `rm.fit' expects a
variable named `"time"' to be present in data; the rest of this
documentation assumes this is the case.  If the time component has
been given a different name, the time optional argument should be
used.  Other functions like `rm.mean' and `rm.plot' do not need the time
optional argument, the time component of the list being normally
identified through the formula.
.PP
The simplest situation occurs when all subjects are observed at the
same time points. In this case time can be a vector. The response
variable is then a matrix with as many columns as elements of time.
This arrangement of the data list is still possible even when some
elements of the response matrix are in fact missing. A typical
situation of this kind occurs when the design of the experiment
expects all subjects to be observed at the same time points, but in
practice some of the subjects were not observed on some of the
scheduled occasions.  NA values can then be inserted in the response
matrix.	`rm.fit' assumes they are missing completely at random.
.PP
A more complicated situation arises when the observation times differ
for all or a large number of the individuals. In this case time must
be a matrix with as many rows as the number of subjects.  The number
of columns in the response and time matrices is equal to the highest
number of repeated measurements among individuals. Rows representing
individuals with fewer measurements are completed with NA values:
observed values can occupy any column in a row, but the same ordering
of observed and NA values must be maintained in every matrix of the
list.  It is recommended that time be set as a matrix only for highly
unbalanced problems; in the other cases, it is convenient to have a
time vector, possibly inserting some NA's in the response variable
matrix.
.PP     
If discrete data are used, the list can represent a table of profiles.
In this case, the response matrix represents the patterns of response
and a vector should be included in the list, representing the counts,
i.e. the replicated observations, for each profile. Typically, such
vector will contain as many elements as the response matrix has rows.
`rm.fit()' and `rm.mean()' expect such vector to be named
`counts'.  If this is the case, the vector is automatically detected
and used by the function and no action is required from the user.  If
the counts vector has been given a different name, the counts}
optional argument should be used.
.PP
Covariates which are constant over time must be represented by vectors
with as many elements as the rows of the response matrix. Time
dependent covariates which are constant across subjects must be
represented by vectors with as many elements as the number of columns
of the response matrix. Covariates varying simultaneously with time
and subject are represented by matrices. In the event that the number
of subjects coincides with the number of replicated observations on
each individual, i.e. the response matrix is a squared matrix, it is
recommended to code time dependent covariates which are constant
across subjects in the form of a matrix with all rows equal.
.PP
Factors need to be declared (see the documentation of `factor',
`as.factor', `is.factor' for details). Since S-PLUS does not allow factors
to be contained in matrices, categorical variables varying with the
time must be contained in vectors. The length of such vectors is equal
to the product of the number of rows and columns of the response
variable matrix. Observations inside the vector should be grouped by
individuals. For example, if we are observing two individuals each of
which shows four replications, vectors of the categorical variables
varying with the time should have eight elements of which the first
fours represent the replicated observations of the first individual
and the second fours those of the second individual. When these
vectors are created by evaluating `factor' (`as.factor') on an existing
matrix `m' satisfying the requirements previously described, the right
call to obtain the vectors in the desired form should be:
.sp
	 `a <- factor(t(m))'.
.SH SUBSET
The `subset' argument, like the terms in `formula', is evaluated in the
context of the data list.  The specific action of the argument is as
follows: before any evaluation starts, the appropriate subset of cases
identified by the condition is extracted. Remember that the correct
syntax for a multiple logical statement is of the type:
.sp
     `variable 1 == "a" & variable2 > 3 | variable3 == "c"'
.SH OPTIMIZATION
Unless `cov.struct="indept"' the model is fit using `nlminb'
(Nonlinear Minimization subject to Box Constraints).  See the
documentation of the function for details.
.SA
`rm.object', `rm.mean', `rm.plot', `ls.smooth', `ksmooth'
.EX
rm.fit(y~x, data = citrate.list)

rm.fit(y~factor(time), data = citrate.list, cov.struct = "AR(1)+random")

rm.fit(y~age*sex,PR.list,cov.struct="I+random",random.effects=y~age)
.WR
