.BG
.FN rm.mean
.TL
Smoothing plots
.DN
Interactive graphical function to analyze mean profiles (possibly smoothed) and their
confidence bands in a repeated measurements context
.CS
rm.mean(formula, data, smoothing, control=NULL, batch=NULL, subset, given.values, conf.int=F,
timelab, ylab, timelim, ylim, ...)
.RA
.AG formula
a formula expression as for regression models of the form `response ~
time (| covariates)'.  At most two covariates can be used in the
predictor. See the documentation of formula for details.
.AG data
a data list in which to interpret the variables occurring in the
formula. List elements are identified by names.  NA values are
allowed. See the documentation of `rm.fit' for details.
.OA
.AG smoothing
label identifying the method of smoothing to be used. If missing, no
smoothing is carried out. Possible smoothing functions are ksmooth,
identified by `"ksmooth"', and `rm.loess', identified by `"loess"'.
See the S-PLUS documentation for details.
.AG control
function which allows users to set options for the smoothers.
Available functions are `loess.ctrl()' and `ksmooth.ctrl()'.
Arguments of the functions can be any of the input options parameters
of the corresponding smoothers. Components that are not set will be
replaced by their defaults within `rm.mean'.
.AG batch
list allowing a batch use (avoiding graphical interaction) of `rm.mean'. 
Elements in list contain declarations of the levels of the covariates
to be used to construct the mean profiles. Declarations are of the
type `expression(covariate1 == "a")'
.AG subset
expression stating which subset of the data rows should be used in the
plot and fit. This is a logical statement of the type `covariate1 ==
"a" & covariate 2 > x' which identifies the observations to be
selected.  The default is `"all observations"'.
.AG given.values
a numeric vector, character vector, or two-column matrix that
specifies the given values when there is one given numerical
predictor, or a list of two such objects when there are two. If
missing, reasonable things happen.
.AG se
if `TRUE', add confidence intervals to the output.
.AG coverage
confidence level of the limits as a decimal fraction. By default, the
0.95 pointwise confidence interval is computed.
.AG timelab
time axis labels. If missing, the name of the variable in `formula' is
used.
.AG ylab
response axis labels. If missing, the name of the variable in `formula'
is used.
.AG timelim
approximate minimum and maximum values to be put on time axis.
.AG ylim
approximate minimum and maximum values to be put on the response axis
.AG panel
a flag that determines the method of plotting on the graphical
window. The default is `FALSE'.
.AG ...
Graphical parameters may also be supplied as arguments to this
function (see `par').
.RT
a list representing the mean profiles produced by the interaction or
the batch use.  Elements of the list are matrices. The first row in
each matrix presents the grandmean profile values; any additional row
presents a sub-profile identified by the interaction.  Row labels are
used to help identify profiles.  If quitting in single mode, the last
active profile is included in the output matrices; if quitting in
incremental mode, the last highlighted profiles (or those highlighted
from the last single-incremental switch) are included. The matrices
have as many rows as the number profiles on the plot window (including
the grandmean) and a number of columns corresponding to the most
frequent replicated observation among the selected profiles.  The
following matrices are present in the list:
.SH DEFAULT VALUES
.AG call
call to the function.
.AG time
times of observations of the profiles values.
.AG fit
 fitted values of the mean profiles.
.SH OPTIONAL VALUES
If `conf.int = T' the following matrices are also present:
.AG  upper
upper values of the 95% confidence bounds for the mean profiles.
.AG  lower
lower values of the 95% confidence bounds for the mean profiles.
.SE
a plot of the grand mean profile and its 95% confidence band are
produced on the current graphics device.  Dialog boxes showing
scrolling lists of all the levels of the covariates (when used) are
also presented. Levels in the scrolling lists can be interactively
`selected' and `deselected'.
.DT
.SH LAYOUT
There are two main layouts for `rm.mean()', depending on the value of
the flag panel. If `panel' is `TRUE', then a matrix of dependence
panels is created with as many panels as the number of levels for the
first conditioning variable. On each dependence panel, mean profiles
are plotted for the subjects showing the active selection for the
covariate(s). The general mean profile is retained on each dependence
panel. If `panel' is `FALSE', then a single plotting window is created,
showing the general mean profile and the mean profiles activated
through interaction.
.SH DIALOG BOXES
A dialog box appears whenever:
(a) `panel' is  `FALSE' and covariates appear in  `formula';
(b) `panel' is  `TRUE' and  two covariates appear in  `formula'.
.br
The upper part of the box contains a menu showing the levels of the
covariates used in the call to the function.  Labels of the covariates
levels as declared in the data list are used to identify the levels;
intervals are used for numerical covariates, when not otherwise stated
(see `given.values' and `co.intervals()').  Two different interactive
options can be chosen when panel is `FALSE': `Incremental scan' or
`Single scan'. A radiobox contains the selection for the interactive
mode; the active mode is shown by the pressed button.  The selection
can be toggled at any time during interaction simply by clicking on
the desired option and the two modes may be switched any number of
times.
.SH GRAPHICAL INTERACTION
Clicking on a level displays the mean profile and possibly the
confidence band of the subset of cases (if any) which present the
selected level.  If this subset is empty, a new interactive selection
is expected from the user. If two covariates are used, cases that
satisfy the active selections on both the covariates are used to
compute the highlighted mean profile.  When panel is FALSE, if a new
selection is activated in incremental mode, the mean profile of all
the cases satisfying the new selection is superimposed to the existing
plot; in the single mode, only the last selection is plotted.  No
graphical interaction is possible if covariates are not used.
.SA
`rm.plot', `rm.fit', `rm.loess', `ksmooth'.
.EX
rm.mean(y~time|x, data=citrate.list)

rm.mean(y~time|x, data = citrate.list, smoothing = "loess",
	control=loess.ctrl(span=1/5), se =T)

rm.mean(y~time|x, data = citrate.list, smoothing = "loess",
	batch = list(expression(x == "yes"),
	expression(x == "no")))
.WR

