".First.lib"<-
function(lib.loc, section)
{
	if(version$major == 3) {
		if(version$minor < 2)
			stop("This library needs S-Plus 3.2 or later"
				)
		else sec <- "sp3"
	}
	if(version$major == 4)
		sec <- "sp4"
	dyn.load(paste(lib.loc, section, sec, "rmtools.obj", sep = 
		"/"))
	invisible()
}
"anova.rm"<-
function(obj)
{
	class(obj) <- NULL
	response <- obj$response
	obj <- obj$anova.aux
	cs <- obj$cov.struct
	if(cs == "I+random" | cs == "AR(1)+random" & obj$p.cov > 1)
		stop("Analysis of deviance not applicable")
	heading <- c("Analysis of Deviance Table", paste(
		"\nResponse: ", response, "\n", sep = ""))
	factor.columns <- obj$deviances$dev.comp[[1]]	
	#lista delle colonne assegnate alle espl
	factor.names <- names(factor.columns)
	n.unbalanced <- obj$n.unbalanced
	if(n.unbalanced != 0)
		cat("Warning Message:  fitted data are unbalanced; \nthis might invalidate anova results, depending \non the degree of unbalancing.\n\n"
			)
	syy <- obj$deviances$syy.omega
	sxy <- obj$deviances$sxy.omega
	sxx <- obj$deviances$sxx.omega
	Q <- obj$deviances$Q
	Q2 <- obj$deviances$Q2
	NT <- sum(obj$ti.repl)	
	#total n. of observations effectively used
	N <- length(obj$ti.repl)	#n. of subjects effectively used 
	Time <- NT/N	#estimated n. of replications 
	p <- length(factor.columns)	#n. of fixed effects
	ps <- obj$n.subj.specific	#n. of fixed subj. spec. effects
	if(!is.na(match("(Intercept)", factor.names)))
		ps <- ps + 1
	res <- c(syy, rep(0, p))
	dfres <- c(rep(NA, p))
	col <- c()
	for(i in 1:p) {
		col <- c(col, factor.columns[[i]])
		res[i + 1] <- syy - t(matrix(sxy[col,  ])) %*% solve(
			sxx[col, col]) %*% matrix(sxy[col,  ])
		dfres[i] <- length(factor.columns[[i]])
	}
	dev.subj <- (Q * (Time - 1) + Q2)/Time
	if(cs == "I+random" | cs == "AR(1)+random") {
		dev.subj <- (Q * (Time - 1) + Q2)/Time
		dev.res <- Q2 - dev.subj
		terms <- c(factor.names, "Subject random effects", 
			"Residual", "Total")
		dfres <- c(dfres, N - ps, NT - sum(c(dfres, N - ps)), 
			NT)
		dev <- c( - diff(res), dev.subj, dev.res)
	}
	else {
		dev.res <- Q2
		dev.subj <- NA
		terms <- c(factor.names, "Residual", "Total")
		dfres <- c(dfres, NT - sum(dfres), NT)
		dev <- c( - diff(res), dev.res)
	}
	aod <- data.frame(Terms = terms, Df = dfres, "Sum of Sq" = c(
		dev, sum(dev)), "Mean Sq" = c(dev, sum(dev))/dfres)
	return(as.anova(aod, heading))
}
"fitted.rm"<-
function(rm.object)
{
#
# define the position of an element in a list
#
	pos <- function(el, list)
	for(i in 1:length(list))
		if(el == list[[i]]) return(i)	#
	class(rm.object) <- NULL
	data.list <- rm.object$data
	fitted.values <- rm.object$fitted.values
	response <- rm.object$response
	data.list[[pos(response, names(data.list))]] <- NULL
	data.list$fitted.values <- fitted.values
	print(fitted.values)
	invisible(data.list)
}
"ksmooth.ctrl"<-
function(kernel = "box", bandwidth = 0.5, n.points = 20, ...)
return(list(kernel = kernel, bandwidth = bandwidth, n.points = 
	n.points, ...))
"loess.ctrl"<-
function(span = 2/3, degree = 1, family = c("symmetric", "gaussian"), 
	evaluation = 50, coverage = 0.95, ...)
return(list(span = span, degree = degree, family = family, evaluation
	 = evaluation, coverage = coverage, ...))
"print.rm"<-
function(x, ...)
{
	if(!is.null(cl <- x$call)) {
		cat("Call:\n")
		dput(cl)
	}
	class(x) <- NULL
	coef <- x$coefficients
	nas <- is.na(coef[, 1])
	cnames <- names(coef[, 1][!nas])
	coef <- matrix(rep(coef[, 1][!nas], 3), ncol = 3)
	coef[, 2] <- x$s.e[, 1][!nas]
	coef[, 3] <- round(coef[, 1]/coef[, 2], 3)
	dimnames(coef) <- list(cnames, c("Value", "Std. Error", 
		"t value"))
	if(x$family != "gaussian")
		coef <- coef[ - dim(coef)[[1]],  ]
	cat("\nCoefficients:\t\n")
	print(coef)
	cat("\nLog likelihood: ", round(x$log.likelihood, 4), "\n\n")
	cat("Message: ", x$message, "\n")
	invisible()
}
"residuals.rm"<-
function(rm.object)
{
#
# define the position of an element in a list
#
	pos <- function(el, list)
	for(i in 1:length(list))
		if(el == list[[i]]) return(i)	#
	class(rm.object) <- NULL
	data.list <- rm.object$data
	residuals <- rm.object$residuals
	response <- rm.object$response
	data.list[[pos(response, names(data.list))]] <- NULL
	data.list$residuals <- residuals
	print(residuals)
	invisible(data.list)
}
"rm.fit"<-
function(formula = formula(data), data, family = "gaussian", 
	cov.struct = "I", random.effects, time, link = "log", subset, 
	start = NULL, trace = T, contrasts = NULL, control = 
	nlminb.control(eval.max = 50, x.tol = 1e-006, rel.tol = 
	1e-006), ...)
{
# *****************DEFINITION OF INTERNAL FUNCTIONS ******************
# 
# na.action for binary families
#
	na.discrete.replace <- function(frame, n.cases, n.times, 
		family)
	{
		check <- function(pos, n, t)
		{
			distance.between.na <- diff(seq(1, n * t)[!
				pos])
			profile.membership <- diff(rep(1:n, rep(t, n)
				)[!pos])
			return(any(distance.between.na > 2 & 
				profile.membership == 0))
		}
		vars <- names(frame)
		names(vars) <- vars
		for(j in 1:length(vars)) {
			x <- frame[[j]]
			pos <- is.na(x)
			if(any(pos))
				if(j == 1) {
				  if(check(pos, n.cases, n.times) & 
				    family == "binary")
				    cat(
				      "Warning Message: Condition on NA's not respected:\nresults might be inaccurate\n"
				      )
				  x[pos] <- -1
				}
				else if(family == "binary")
				  stop(
				    "NA's on covariates not allowed")
				else x[pos] <- x[1]
			frame[[j]] <- x
		}
		frame
	}
#
# compute the fitted values for variuos families of distributions
#
	various.fitted <- function(Fitted, family, link.option)
	{
		if(family == "binary")
			return(1/(1 + care.exp( - Fitted)))
		else if(family == "poisson")
			if(link.option == "log")
				return(care.exp(Fitted))
			else return(Fitted)
		else return(Fitted)
	}
#
# compute numerical second derivatives
#
	num.info <- function(coefficients, FUN = 
		"minus.gradient.binloglik", X, data, info = F, link)
	{
		FUN <- get(FUN, inherit = T)
		values <- FUN(coefficients, X, data, F, link)
		p <- length(values)
		Info <- matrix(0, p, p)
		h <- rep(0, p)
		delta <- cbind((abs(coefficients) + 1e-012) * 0.0001, 
			rep(1e-012, p))
		delta <- apply(delta, 1, max)
		for(i in 1:p) {
			h[i] <- delta[i]
			new.values <- FUN(coefficients + h, X, data, 
				info, link)
			Info[, i] <- (new.values - values)/delta[i]
			h[i] <- 0
		}
		Info
	}
# 
# compute values of parameters given the values of nuisance parameters  
#
	coefficienti <- function(rho, delta, X, data, FF = NULL, 
		vect.time, trace, contrast = NULL, which.columns = 
		NULL, dev = F, ...)
	{
#
# FF is the (k x T) matrix entering in the covariance
#
		R.matrix <- function(rho, delta, time, Fi)
		{
			log.det <- function(x)
			sum(log(eigen(x)$values))
			lt <- length(time)
			d <- diff(time) * 2
			Omega.inv <- matrix(0, lt, lt)
			Omega.inv[1, 1] <- 1/(1 - rho^(d[1]))
			Omega.inv[lt, lt] <- 1/(1 - rho^(d[lt - 1]))
			for(i in 2:lt) {
				if(i < lt)
				  Omega.inv[i, i] <- (1 - rho^(d[i]) * 
				    rho^(d[i - 1]))/((1 - rho^(d[i])) *
				    (1 - rho^(d[i - 1])))
				Omega.inv[i, i - 1] <-  - rho^(0.5 * 
				  d[i - 1])/(1 - rho^(d[i - 1]))
				Omega.inv[i - 1, i] <- Omega.inv[i, i -
				  1]
			}
			log.Omega.det <- sum(log(1 - rho^d))
			if(is.null(Fi)) {
				omega <- Omega.inv %*% rep(1, lt)
				temp <- delta/(1 + delta * sum(omega)
				  )
				R.inv <- Omega.inv - temp * crossprod(
				  t(omega))
				log.R.det <- log.Omega.det + log(1 + 
				  delta * sum(omega))
			}
			else {
# delta arrives reparameterized as log(delta)
				k <- length(delta)
				Delta <- matrix(0, k, k)
				diag(Delta) <- exp(delta)
				Fi <- matrix(Fi, nrow = k)
				AA <- Delta + Delta %*% Fi %*% 
				  Omega.inv %*% t(Fi) %*% Delta
				if(all(AA == 0))
				  R.inv <- Omega.inv
				else R.inv <- Omega.inv - Omega.inv %*% 
				    t(Fi) %*% Delta %*% solve(AA) %*% 
				    Delta %*% Fi %*% Omega.inv
				log.R.det <- log.Omega.det + log.det(
				  diag(rep(1, k)) + Fi %*% Omega.inv %*% 
				  t(Fi) %*% Delta)
			}
			return(list(R.inv, log.R.det, Omega.inv))
		}
# 			
		if(trace)
			if(is.null(FF)) {
				cat(paste(round(rho, 4), "\t", round(
				  delta, 4)))
			}
			else {
				d <- paste(round(exp(delta), 4), 
				  collapse = " ")
				cat(paste(round(rho, 4), "\t", d))
			}
		p <- dim(X)[2]
		ti.repl <- data[[1]]
		n.cases <- length(ti.repl)
		cumti.repl <- cumsum(ti.repl)
		data.list <- data[[2]]
		sxx <- sxx.omega <- matrix(0, p, p)
		sxy <- sxy.omega <- matrix(0, p, 1)
		syy <- syy.omega <- Q <- Q2 <- dev.R <- 0
		if(vect.time & is.null(FF))
			Ri <- R.matrix(rho, delta, data.list[1:
				ti.repl[1], 2], FF)
		log.R.det <- 0
		k1 <- 1
		for(i in 1:n.cases) {
			k2 <- cumti.repl[i]
			Xi <- X[k1:k2,  ]
			if(!is.null(FF))
				Fi <- t(FF[k1:k2,  ])
			else Fi <- NULL
			yi <- as.matrix(data.list[k1:k2, 1])
			if(!vect.time | !is.null(FF))
				Ri <- R.matrix(rho, delta, data.list[
				  k1:k2, 2], Fi)
			sxx <- sxx + t(Xi) %*% Ri[[1]] %*% Xi
			sxy <- sxy + t(Xi) %*% Ri[[1]] %*% yi
			syy <- syy + (t(yi) %*% Ri[[1]] %*% yi)[1, 1]
			log.R.det <- log.R.det + Ri[[2]]
			k1 <- k2 + 1
		}
		sxx.inv <- solve(sxx)
		beta <- sxx.inv %*% sxy
		beta.sxx.beta <- (t(sxy) %*% sxx.inv %*% sxy)[1, 1]
		sum.Di <- syy - beta.sxx.beta
		sigma2 <- sum.Di/sum(ti.repl)
		if(dev) {
			k1 <- 1
			for(i in 1:n.cases) {
				k2 <- cumti.repl[i]
				Xi <- X[k1:k2,  ]
				if(!is.null(FF))
				  Fi <- t(FF[k1:k2,  ])
				else Fi <- NULL
				yi <- as.matrix(data.list[k1:k2, 1])
				if(!vect.time | !is.null(FF))
				  Ri <- R.matrix(rho, delta, 
				    data.list[k1:k2, 2], Fi)
				sxx.omega <- sxx.omega + t(Xi) %*% Ri[[
				  3]] %*% Xi
				syy.omega <- syy.omega + (t(yi) %*% 
				  Ri[[3]] %*% yi)[1, 1]
				sxy.omega <- sxy.omega + t(Xi) %*% Ri[[
				  3]] %*% yi
				mu <- Xi %*% beta
				Q <- Q + (t(yi - mu) %*% (Ri[[3]] - 
				  Ri[[1]]) %*% (yi - mu))[1, 1]
				Q2 <- Q2 + (t(yi - mu) %*% Ri[[3]] %*% (
				  yi - mu))[1, 1]
				k1 <- k2 + 1
			}
			df <- c(sum(ti.repl), n.cases)
			deviances <- list(beta.sxx.beta = 
				beta.sxx.beta, Q = Q, Q2 = Q2, 
				syy.omega = syy.omega, sxy.omega = 
				sxy.omega, sxx.omega = sxx.omega, syy
				 = syy, sxx = sxx, sxy = sxy, 
				dev.comp = which.columns)
			return(list(sigma2, log.R.det, sum.Di, beta, 
				sxx.inv, deviances))
		}
		else return(list(sigma2, log.R.det, sum.Di, beta, 
				sxx.inv))
	}
#########################################################################
########################################################################
# ------------------------------------------------------------------- 
	minus.gradient.binloglik <- function(param, X, data, info = F,
		link = "logit")
	{
#
		minus.gradient <- function(param, dataset, link = 
			"logit")
		{
			y <- dataset$y
			y[is.na(y)] <- (-1)
			y <- as.integer(y)
			n <- length(y)
			npar <- length(param)
			beta <- as.double(param[1:(npar - 1)])
			log.psi <- as.double(param[npar])
			theta <- work <- as.double(rep(0, n))
			g.beta <- d.beta0 <- d.beta1 <- der <- 
				as.double(rep(0, npar - 1))
			g.logpsi <- as.double(0)
			d.beta <- matrix(as.double(0), 3, npar - 1)
			x <- matrix(as.double(dataset$x), nrow = n, 
				ncol = npar - 1)
			result <- .Fortran("bin_grad",
				g.beta,
				g.logpsi,
				beta,
				log.psi,
				npar,
				x,
				y,
				theta,
				work,
				der,
				d.beta,
				d.beta0,
				d.beta1,
				n)
			return(c( - result[[1]],  - result[[2]]))
		}
		ti.repl <- data[[1]]
		cumti.repl <- cumsum(ti.repl)
		n.cases <- length(ti.repl)
		p <- length(param)
		y <- data[[2]]
		counts <- data[[3]]
		neg.grad <- 0
		Info <- matrix(0, p, p)
		k1 <- 1
		for(i in 1:n.cases) {
			k2 <- cumti.repl[i]
			Xi <- X[k1:k2,  ]
			yi <- y[k1:k2]
			u <- minus.gradient(param, list(x = Xi, y = 
				yi), link) * counts[i]
			neg.grad <- neg.grad + u
			k1 <- k2 + 1
		}
		if(info)
			return(list(neg.grad, Info))
		else return(neg.grad)
	}
# ------------------------------------------------------------------- 
	minus.gradient.poissonloglik <- function(param, X, data, info
		 = F, link)
	{
#
		minus.gradient <- function(param, dataset, link)
		{
			y <- dataset$y
			y[is.na(y)] <- (-1)
			y <- as.integer(y)
			n <- length(y)
			npar <- length(param)
			beta <- as.double(param[1:(npar - 1)])
			rho <- as.double(param[npar])
			theta <- work <- as.double(rep(0, n))
			grad <- as.double(rep(0, npar))
			x <- matrix(as.double(dataset$x), nrow = n, 
				ncol = npar - 1)
			m <- max(y)
			fact <- rep(1, m + 1)
			if(m > 0)
				for(i in 2:m + 1)
				  fact[i] <- fact[i - 1] * (i - 1)
			fact <- as.double(fact)
			link <- as.integer(link)
			result <- .Fortran("pss_grad",
				grad,
				beta,
				rho,
				npar,
				x,
				y,
				theta,
				work,
				n,
				fact,
				link)
			return(c( - result[[1]]))
		}
		ti.repl <- data[[1]]
		cumti.repl <- cumsum(ti.repl)
		n.cases <- length(ti.repl)
		p <- length(param)
		y <- data[[2]]
		counts <- data[[3]]
		neg.grad <- 0
		Info <- matrix(0, p, p)
		k1 <- 1
		for(i in 1:n.cases) {
			k2 <- cumti.repl[i]
			Xi <- X[k1:k2,  ]
			yi <- y[k1:k2]
			u <- minus.gradient(param, list(x = Xi, y = 
				yi), link) * counts[i]
			neg.grad <- neg.grad + u
			k1 <- k2 + 1
		}
		if(info)
			return(list(neg.grad, Info))
		else return(neg.grad)
	}
#########################################################################
#
# define the likelihood function for different covariance structure
#
	logL.rho.random <- function(parameters, X, data, FF, 
		vect.time, trace)
	{
		coeff <- get("coefficienti", inherit = T)
		param <- coeff(parameters[1], parameters[-1], X, data,
			FF, vect.time, trace)
		loglik <- log(param[[1]]) * sum(data[[1]]) + param[[2
			]]
		if(trace)
			cat(paste("\t", -0.5 * loglik), "\n")
		return(loglik)
	}
#
	logL.delta.random <- function(parameters, X, data, FF, 
		vect.time, trace)
	{
		coeff <- get("coefficienti", inherit = T)
		param <- coeff(0, parameters, X, data, FF, vect.time, 
			trace)
		loglik <- log(param[[1]]) * sum(data[[1]]) + param[[2
			]]
		if(trace)
			cat(paste("\t", -0.5 * loglik), "\n")
		return(loglik)
	}
#
	logL.rho.delta <- function(parameters, X, data, vect.time, 
		trace)
	{
		coeff <- get("coefficienti", inherit = T)
		param <- coeff(parameters[1], parameters[2], X, data, 
			NULL, vect.time, trace)
		loglik <- log(param[[1]]) * sum(data[[1]]) + param[[2
			]]
		if(trace)
			cat(paste("\t", -0.5 * loglik), "\n")
		return(loglik)
	}
#
	logL.rho <- function(parameters, X, data, vect.time, trace)
	{
		coeff <- get("coefficienti", inherit = T)
		param <- coeff(parameters[1], 0, X, data, NULL, 
			vect.time, trace)
		loglik <- log(param[[1]]) * sum(data[[1]]) + param[[2
			]]
		if(trace)
			cat(paste("\t", -0.5 * loglik), "\n")
		return(loglik)
	}
#
	logL.delta <- function(parameters, X, data, vect.time, trace
		)
	{
		coeff <- get("coefficienti", inherit = T)
		param <- coeff(0, parameters, X, data, NULL, 
			vect.time, trace)
		loglik <- log(param[[1]]) * sum(data[[1]]) + param[[2
			]]
		if(trace)
			cat(paste("\t", -0.5 * loglik), "\n")
		return(loglik)
	}
#
	logL <- function(X, data, vect.time, trace, which.columns)
	{
		coeff <- get("coefficienti", inherit = T)
		param <- coeff(0, 0, X, data, NULL, vect.time, trace
			 = F, which.columns = which.columns, dev = T)
		return(param)
	}
#
# compute loglik for binary response
#
	logL.binary <- function(parameters, X, data, trace, link = 
		"logit")
	{
#
		minus.loglik <- function(param, dataset, trace, link
			 = "logit")
		{
			y <- dataset$y
			y[is.na(y)] <- (-1)
			y <- as.integer(y)
			n <- length(y)
			npar <- length(param)
			beta <- as.double(param[1:(npar - 1)])
			log.psi <- as.double(param[npar])
			theta <- work <- as.double(rep(0, n))
			x <- matrix(as.double(dataset$x), nrow = n, 
				ncol = npar - 1)
			logL <- as.double(0)
			results <- .Fortran("bin_llik",
				logL,
				beta,
				log.psi,
				npar,
				x,
				y,
				theta,
				work,
				n)
			logL <- results[[1]]
			return( - logL)
		}
		if(trace)
			cat(paste(round(parameters[length(parameters)
				], 4), "\t"))
		ti.repl <- data[[1]]
		cumti.repl <- cumsum(ti.repl)
		n.cases <- length(ti.repl)
		y <- data[[2]]
		counts <- data[[3]]
		neg.logL <- 0
		k1 <- 1
		for(i in 1:n.cases) {
			k2 <- cumti.repl[i]
			Xi <- X[k1:k2,  ]
			yi <- y[k1:k2]
			z <- minus.loglik(parameters, dataset = list(
				x = Xi, y = yi), trace, link = 
				"logit")
			neg.logL <- neg.logL + counts[i] * z
			k1 <- k2 + 1
		}
		if(trace)
			cat(paste("\t",  - neg.logL), "\n")
		return(neg.logL)
	}
#
# compute loglik for poisson response
#
	logL.poisson <- function(parameters, X, data, trace, link)
	{
#
		minus.loglik <- function(param, dataset, trace, link
			)
		{
			y <- dataset$y
			y[is.na(y)] <- (-1)
			y <- as.integer(y)
			n <- length(y)
			npar <- length(param)
			beta <- as.double(param[1:(npar - 1)])
			rho <- as.double(param[npar])
			theta <- work <- as.double(rep(0, n))
			link <- as.integer(link)
			x <- matrix(as.double(dataset$x), nrow = n, 
				ncol = npar - 1)
			logL <- as.double(0)
			m <- max(y)
			fact <- rep(1, m + 1)
			if(m > 0)
				for(i in 2:m + 1)
				  fact[i] <- fact[i - 1] * (i - 1)
			fact <- as.double(fact)
			results <- .Fortran("pss_llik",
				logL,
				beta,
				rho,
				npar,
				x,
				y,
				theta,
				work,
				n,
				fact,
				link)
			logL <- results[[1]]
			return( - logL)
		}
		if(trace)
			cat(paste(round(parameters[length(parameters)
				], 4), "\t"))
		ti.repl <- data[[1]]
		cumti.repl <- cumsum(ti.repl)
		n.cases <- length(ti.repl)
		y <- data[[2]]
		counts <- data[[3]]
		neg.logL <- 0
		k1 <- 1
		for(i in 1:n.cases) {
			k2 <- cumti.repl[i]
			Xi <- X[k1:k2,  ]
			yi <- y[k1:k2]
			z <- minus.loglik(parameters, dataset = list(
				x = Xi, y = yi), trace, link)
			neg.logL <- neg.logL + counts[i] * z
			k1 <- k2 + 1
		}
		if(trace)
			cat(paste("\t",  - neg.logL), "\n")
		return(neg.logL)
	}
# ******************* MAIN PROGRAM *******************************
#
	call <- match.call()
	vect.time <- F
	if(trace && cov.struct == "indep" && family != "binary" && 
		family != "poisson")
		trace <- F
	if(missing(data) || !is.list(data))
		stop("a data list must be supplied")
	if(is.null(names(data)))
		stop("objects in data list must have a name")
	expr1 <- terms(formula, data)
	expr <- attr(expr1, "variables")
	var.names <- all.vars(expr)
	response <- all.vars(expr)[1]
	if(!missing(random.effects)) {
		expr2 <- terms(random.effects, data)
		expr.cov <- attr(expr2, "variables")
		var.names.cov <- all.vars(expr.cov)
		var.names <- unique(c(var.names, var.names.cov))
	}
	n.cases <- dim(data[[response]])[1]
	n.times <- dim(data[[response]])[2]
	if(missing(time)) {
		if(is.na(match("time", names(data)))) {
			cat("Warning message:\ntime variable missing. Set to be 1:n.replications\n"
				)
			data$time <- 1:n.times
		}
		time <- "time"
	}
	if(any(is.na(match(var.names, names(data)))))
		stop("Variables in formula not contained in the data list"
			)
	if(is.na(match(time, var.names)))
		var.names <- c(var.names, time)
	if(!missing(subset)) {
		id <- eval(substitute(subset), data)
		if(any(id)) {
			for(i in 1:length(data))
				if(is.matrix(data[[i]])) data[[i]] <- 
				    data[[i]][id,  ] else if(n.times == 
				  n.cases)
				  stop(
				    "Subset condition might generate errors.\nPlease, replace time-dependent covariates with matrices in the data list"
				    )
				else if(length(data[[i]]) != n.times
				  )
				  data[[i]] <- data[[i]][id]
		}
		else stop("Subset condition not satisfied by any case"
				)
	}
	final.data <- data
	Y <- matrix(eval(expr[[1]], data), ncol = n.times)	
	#	Y <- eval(expr[[1]],data)
	if(family == "binary")
		if((all(Y[!is.na(Y)] == 1 | Y[!is.na(Y)] == 0)) == F
			)
			stop("Unfeasible values of response variable: must be 0,1,NA"
				)
	if(family == "poisson") {
		y1 <- Y[!is.na(Y)]
		if((all(y1 >= 0) | all(y1[y1 != 0] == as.integer(y1[
			y1 != 0]))) == F)
			stop("Unfeasible values of response variable: must be non negative integers or NA"
				)
	}
	n.cases <- dim(Y)[1]
	n.tot <- n.cases * n.times
	if(family == "binary" | family == "poisson") {
		if(is.na(match("counts", names(data))))
			counts <- data$counts <- rep(1, n.cases)
		else counts <- data$counts
		n.counts <- length(counts)
		var.names <- c(var.names, "counts")
	}
	data <- data[var.names]
	n.var <- length(data)
	Time <- data[[time]]	# modificato
	if(is.vector(Time)) {
		Time <- matrix(rep(Time, n.cases), nrow = n.cases, 
			byrow = T)
	}
#
# *********************    data transformation      **********************
# elements in <data> are transformed into vectors of length  n.tot
#
	time.specific <- rep(T, n.var)
	for(i in 1:n.var) {
		if(is.matrix(data[[i]])) {
			data[[i]] <- as.vector(t(data[[i]]))
		}
		else {
			l <- length(data[[i]])
			if(l != n.tot)
				if(l == n.cases) {
				  time.specific[i] <- F
				  data[[i]] <- rep(data[[i]], rep(
				    n.times, n.cases))
				}
				else data[[i]] <- rep(data[[i]], 
				    n.cases)
		}
	}
	n.subj.specific <- length(time.specific[time.specific == F])	
	#
# ********** design matrices creation *******************
#
	data <- temp <- as.data.frame(data)
	id.not.na <- rep(T, n.tot)
	if(family == "binary" | family == "poisson") {
		if(n.counts != n.cases) stop(
				"number of counts not correct")	
	# define a plausible starting point for the optimizer if not given
		data1 <- na.omit(data)
		data1.resp <- data1[, response]
		if(family == "binary")
			data1.resp <- log((data1.resp + 0.5)/(1.5 - 
				data1.resp))
		else if(link == "log")
			data1.resp <- log(data1.resp + 1)
		data1[, c(response)] <- data1.resp
		if(family == "binary")
			init <- 1
		else init <- 0.5
		if(is.null(start))
			start <- c(lm(formula, data1, weights = 
				counts)$coefficients, init)
		else start <- c(lm(formula, data1, weights = counts)$
				coefficients, start)
		data <- na.discrete.replace(data, n.cases, n.times, 
			family)
		ti.repl <- rep(n.times, n.cases)
	}
	else {
		data <- na.omit(data)
		id.not.na[is.na(match(row.names(temp), row.names(data
			)))] <- F
		ti.repl <- c()	
	# vett. del num. di valori != 0 di ogni osservaz. 
		cumti.repl <- cumsum(rep(n.times, n.cases))
		x <- k1 <- 1
		n.unbalanced <- 0
		missing.pattern <- Time[1,  ][id.not.na[1:cumti.repl[
			1]]]
		for(i in 1:n.cases) {
			k2 <- cumti.repl[i]
			id.profile <- id.not.na[k1:k2]
			if(!all(!id.profile)) {
				ti.repl[x] <- length(id.profile[
				  id.profile == T])
				x <- x + 1
				Timei <- Time[i,  ][id.profile]
				if(length(Timei) != length(
				  missing.pattern))
				  n.unbalanced <- n.unbalanced + 1
				else if(any(Timei != missing.pattern)
				  )
				  n.unbalanced <- n.unbalanced + 1
				missing.pattern <- Timei
			}
			k1 <- k2 + 1
		}
	}
	X <- model.matrix(expr1, data, contrasts)
	names.output <- dimnames(X)[[2]]
	which.columns <- list(attr(X, "assign"))
	if(family == "gaussian" && n.unbalanced == 0) vect.time <- T	
	#
# deals with random effects
#
	p.cov <- 0	# number of columns in cov.model
	if(!missing(random.effects)) {
		FF <- model.matrix(expr2, data, contrasts)
		names.nuisance <- dimnames(FF)[[2]]
		p.cov <- dim(FF)[2]	
	#number of columns for the cov.model 
	}
	else FF <- NULL
	sum.ti <- sum(ti.repl)
	if(family == "gaussian") {
		data <- matrix(c(as.vector(t(Y)), as.vector(t(Time))),
			ncol = 2)[id.not.na,  ]
		data <- list(ti.repl, data)
	}
	else {
		data <- as.vector(t(Y))
		data <- list(ti.repl, data, counts)
	}
	if(family == "gaussian") {
		if(trace)
			if(cov.struct != "I")
				cat("\nrho\t delta\t log.likelihood\n"
				  )
		if(cov.struct == "I") {
# this is the independence case
			rho <- NULL
			delta <- NULL
			param <- logL(X, data, vect.time, trace, 
				which.columns)
			temp <- list(objective = log(param[[1]]) * 
				sum(data[[1]]) + param[[2]], message
				 = NULL)
		}
		else if(cov.struct == "AR(1)") {
# this is  AR(1)
			if(is.null(start)) start <- 0
			temp <- nlminb(start = start, obj = logL.rho, 
				lower = -0.9999, upper = 0.9999, data
				 = data, X = X, vect.time = vect.time,
				trace = trace, control = control, ...
				)
			rho <- temp$parameters
			delta <- NULL
			param <- coefficienti(rho, 0, X, data, NULL, 
				vect.time, trace, which.columns = 
				which.columns, dev = T)
		}
		else if(cov.struct == "I+random") {
			if(missing(random.effects)) {
# this is I+compound simmetry
				if(is.null(start)) start <- 0
				temp <- nlminb(start = start, obj = 
				  logL.delta, data = data, X = X, 
				  vect.time = vect.time, trace = 
				  trace, lower = 0, upper = NA, 
				  control = control, ...)
				rho <- NULL
				delta <- temp$parameters
				param <- coefficienti(0, delta, X, 
				  data, NULL, vect.time, trace, 
				  which.columns = which.columns, dev
				   = T)
			}
			else {
# this is I+general random effects 
				if(is.null(start)) start <- log(rep(
				    0.5, p.cov)) else start <- log(
				    start)	
	#  delta riparameterized as log(delta)
				temp <- nlminb(start = start, obj = 
				  logL.delta.random, lower = c(rep( - 
				  Inf, p.cov)), upper = c(rep(NA, 
				  p.cov)), data = data, X = X, FF = 
				  FF, vect.time = vect.time, trace = 
				  trace, control = control, ...)
				rho <- NULL
				delta <- temp$parameters
				param <- coefficienti(0, delta, X, 
				  data, FF, vect.time, trace, 
				  which.columns = which.columns, dev
				   = T)
				k <- length(delta)
				delta <- matrix(exp(delta), k, 1)
				dimnames(delta) <- list(
				  names.nuisance, c(" "))
			}
		}
		else if(cov.struct == "AR(1)+random") {
			if(missing(random.effects)) {
# this is ar(1)+compound simmetry 
				if(is.null(start)) start <- c(0, 0)
				temp <- nlminb(start = start, obj = 
				  logL.rho.delta, lower = c(-0.9999, 
				  0), upper = c(0.9999, NA), data = 
				  data, X = X, vect.time = vect.time, 
				  trace = trace, control = control, 
				  ...)
				rho <- temp$parameters[1]
				delta <- temp$parameters[2]
				param <- coefficienti(rho, delta, X, 
				  data, NULL, vect.time, trace, 
				  which.columns = which.columns, dev
				   = T)
			}
			else {
# this is ar(1)+general random effects
				if(is.null(start)) start <- c(0, log(
				    rep(0.5, p.cov))) else start <- c(
				    start[1], log(start[-1]))	
	#delta riparameterized as log(delta) 
				temp <- nlminb(start = start, obj = 
				  logL.rho.random, lower = c(-0.9999, 
				  rep( - Inf, p.cov)), upper = c(
				  0.9999, rep(NA, p.cov)), data = 
				  data, X = X, FF = FF, vect.time = 
				  vect.time, trace = trace, control
				   = control, ...)
				rho <- temp$parameters[1]
				delta <- temp$parameters[-1]
				param <- coefficienti(rho, delta, X, 
				  data, FF, vect.time, trace, 
				  which.columns = which.columns, dev
				   = T)
				k <- length(delta)
				delta <- matrix(exp(delta), k, 1)
				dimnames(delta) <- list(
				  names.nuisance, c(" "))
			}
		}
		else stop("misspelling of cov.struct or covariance structure \n\t\tnot handled by the program"
				)
		log.lik <- -0.5 * temp$objective
		cat(paste("\t", log.lik, collapse = ""))
		cat("\nConvergence reached.\n")
		message <- temp$message
		deviances <- param[[6]]
		coefficients <- temp <- param[[4]]
		temp[, 1] <- diag(sqrt(param[[1]] * param[[5]]))
		dimnames(temp) <- dimnames(coefficients) <- list(
			names.output, " ")
		covariance <- param[[1]] * param[[5]]
		dimnames(covariance) <- list(names.output, 
			names.output)
		p <- length(coefficients) + 1
		anova.aux <- list(ti.repl = ti.repl, n.subj.specific
			 = n.subj.specific, deviances = deviances, 
			cov.struct = cov.struct, n.unbalanced = 
			n.unbalanced, p.cov = p.cov)
	}
	else if(family == "binary") {
		p <- dim(X)[2] + 1
		if(trace)
			cat("\nlog.odds.ratio\t log.likelihood\n")
		temp <- nlminb(start = start, obj = logL.binary, 
			lower = c(rep( - Inf, p)), upper = c(rep(Inf, 
			p)), gradient = minus.gradient.binloglik, 
			data = data, X = X, trace = trace, control = 
			control, ...)
		coefficients <- temp$parameters
		log.lik <-  - temp$objective
		cat("Convergence reached. Computing the information matrix now\n"
			)
		Info <- num.info(coefficients, 
			"minus.gradient.binloglik", X, data, F, 
			"logit")
		se <- matrix(sqrt(diag(solve(Info))), ncol = 1)
		coefficients <- matrix(coefficients, ncol = 1)
		dimnames(coefficients) <- dimnames(se) <- list(c(
			names.output, "log.odds.ratio"), " ")
		covariance <- solve(Info)
		dimnames(covariance) <- list(c(names.output, 
			"log.odds.ratio"), c(names.output, 
			"log.odds.ratio"))
	}
	else {
		p <- dim(X)[2] + 1
		if(start[p] <= 0)
			start[p] <- 1e-006
		if(trace)
			cat("\nrho\t\t log.likelihood\n")
		if(link == "log")
			link <- 1
		else link <- 0
		temp <- nlminb(start = start, obj = logL.poisson, 
			lower = c(rep( - Inf, p - 1), 1e-006), upper
			 = c(rep(NA, p - 1), 0.9999), gradient = 
			minus.gradient.poissonloglik, data = data, X
			 = X, trace = trace, link = link, control = 
			control, ...)
		coefficients <- matrix(temp$parameters, ncol = 1)
		cat("Convergence reached. Computing the information matrix now\n"
			)
		Info <- num.info(coefficients, 
			"minus.gradient.poissonloglik", X, data, F, 
			link)
		se <- matrix(sqrt(diag(solve(Info))), ncol = 1)
		dimnames(se) <- dimnames(coefficients) <- list(c(
			names.output, "rho"), " ")
		covariance <- solve(Info)
		dimnames(covariance) <- list(c(names.output, "rho"), 
			c(names.output, "rho"))
		if(link == 1)
			link <- "log"
		else link <- "identity"
	}
	Fitted <- rep(NA, n.tot)
	Fitted[id.not.na] <- X %*% coefficients[1:(p - 1)]
	Fitted <- various.fitted(Fitted, family, link)
	Fitted <- matrix(Fitted, nrow = n.cases, ncol = n.times, 
		byrow = T)
	if(family == "gaussian") {
		results <- list(coefficients = coefficients, s.e = 
			temp, covariance = covariance, log.likelihood
			 = log.lik, message = message, data = 
			final.data, n.cases = length(ti.repl), 
			residuals = Y - Fitted, fitted.values = 
			Fitted, response = var.names[1], family = 
			"gaussian", sigma2 = param[[1]], rho = rho, 
			delta = delta, anova.aux = anova.aux)
	}
	else {
		Fitted[is.na(Y)] <- NA
		results <- list(coefficients = coefficients, s.e = se,
			covariance = covariance, log.likelihood =  - 
			temp$objective, message = temp$message, data
			 = final.data, n.cases = n.cases, residuals
			 = Y - Fitted, fitted.values = Fitted, 
			response = var.names[1], family = family,  )
	}
	results$call <- call
	results$terms <- expr1
	class(results) <- "rm"
	invisible(results)
}
"rm.loess"<-
function(x, y, span = 2/3, degree = 1, family = c("symmetric", 
	"gaussian"), evaluation = 50, coverage = 0.95)
{
	predict <- function(object, newdata, loess.frame)
	{
		if(missing(newdata))
			return(fitted(object))
		new.x <- matrix(newdata, ncol = 1)
		dimnames(new.x) <- list(NULL, object$predictors$
			names.predictors)
		new.composite <- rep("all.numeric", length(newdata))
		new.names.levels <- "all.numeric"
		new.x <- sweep(new.x, 2, object$surface$divisor, "/")
		ranges <- sweep(object$predictors$ranges, 3, object$
			surface$divisor, "/")
		y <- loess.frame$y
		x <- sweep(loess.matrix(loess.frame)$x, 2, object$
			surface$divisor, "/")
		m <- dim(new.x)[1]
		d <- dim(new.x)[2]
		names.new.x <- dimnames(new.x)[[2]]
		fit <- rep(NA, m)
		se <- rep(NA, m)
		for(f in new.names.levels) {
			which.new <- new.composite == f
			which <- object$predictors$composite == f
			if(object$control$surface == "direct") {
				evaluation <- loess.dfitse(y[which], 
				  x[which,  , drop = F], new.x[
				  which.new,  , drop = F], object$
				  errors$weights[which], object$
				  errors$robust[which], object$errors$
				  family, object$surface$span, object$
				  surface$degree, object$surface$
				  new.parametric, object$surface$
				  new.drop.square, object$inference$s
				  )
				fit[which.new] <- evaluation$fit
				se[which.new] <- evaluation$se.fit
			}
			else {
				inside <- matrix(F, nrow = sum(
				  which.new), ncol = d)
				dimnames(inside) <- list(NULL, 
				  dimnames(new.x)[[2]])
				for(variable in names.new.x) {
				  data <- new.x[which.new, variable]
				  inside[, variable] <- (data <= 
				    ranges[f, "max", variable]) & (
				    data >= ranges[f, "min", variable
				    ])
				}
				inside <- inside %*% rep(1, d) == d
				if(sum(inside) > 0) {
				  fit[which.new][inside] <- 
				    loess.ifit(object$surface$
				    interpolator[[f]], new.x[
				    which.new,  , drop = F][inside,  ,
				    drop = F], object$surface$
				    new.parametric)
				  se[which.new][inside] <- loess.ise(
				    y[which], x[which,  , drop = F], 
				    new.x[which.new,  , drop = F][
				    inside,  , drop = F], object$
				    errors$weights[which], object$
				    surface$span, object$surface$
				    degree, object$surface$
				    new.parametric, object$surface$
				    new.drop.square, object$control$
				    cell * object$surface$span, 
				    object$inference$s)
				}
			}
		}
		if(!is.null(a <- attr(newdata, "out.attrs")$dim)) {
			fit <- array(fit, a, dimnames = attr(newdata, 
				"out.attrs")$dimnames)
			se <- array(se, a, dimnames = attr(newdata, 
				"out.attrs")$dimnames)
		}
		return(list(fit = fit, se.fit = se, residual.scale = 
			object$inference$s, df = object$inference$
			one.delta^2/object$inference$two.delta))
	}
	newdata <- seq(min(x), max(x), length = evaluation)
	object <- loess(formula = y ~ x, data = data.frame(y = y, x
		 = x), span = span, degree = degree, family = 
		match.arg(family))
	loess.frame <- model.frame(data.frame(y = y, x = x))
	fit <- predict(object, newdata, loess.frame)
	temp <- pointwise(fit, coverage)
	return(list(newdata, temp))
}
"rm.mean"<-
function(formula, data, smoothing, control = NULL, subset, counts, 
	given.values, batch = NULL, se = F, coverage = 0.95, rsd = F, 
	timelab, ylab, timelim, ylim, panel = T, ...)
{
#############################################################################
#	      	  *** DEFINITION OF INTERNAL FUNCTIONS ***		    #  
#############################################################################
	message.interactive <- function()
	cat("Control passing to `Conditioning levels` menu.\nClick on 'OK' after selection, `Quit` to stop.\n"
		)	#
# closing message
#
	closing.message <- function()
	{
		cat("Warning message:\nOne group of profiles only found to be usable.\nGraphical interaction disactivated.\n"
			)
		invisible()
		return()
	}
#
# return a suitable matrix
#
	tas.matrix <- function(x)
	{
		if(is.matrix(x))
			return(x)
		else return(matrix(x, nrow = 1, byrow = T))
	}
#	
# find cases corresponding to levels of two covariates
#
	select.cases <- function(longer, shorter, which.levels.longer,
		which.levels.shorter, n.cases, panel = F)
	{
		pick <- function(G, g, level)
		G >= g[level, 1] & G <= g[level, 2] & !is.na(G)
		if(panel)
			id <- rep(0, n.cases)
		else id <- rep(1, n.cases)
		if(is.null(shorter))
			id[pick(longer$given, longer$intervals, 
				which.levels.longer)] <- 3
		else {
			temp <- temp1 <- rep(1, n.cases)
			temp[pick(longer$given, longer$intervals, 
				which.levels.longer)] <- 3
			temp1[pick(shorter$given, shorter$intervals, 
				which.levels.shorter)] <- 3
			temp3 <- (temp == 3) & (temp1 == 3)	
	#	   if(panel){
#		id[temp==3] <- 1
#		id[temp3] <- 3
#	   }  
#	   else 
			id[temp3] <- 3
		}
		return(id)
	}
#
# compute mean profiles and confidence intervals 
#
	mean.points <- function(response, repl, counts, smoothing, 
		control, rsd, m, coverage, ...)
	{
#		repl <- as.vector(t(repl))
		if(!is.matrix(response)) response <- matrix(response, 
				nrow = 1)
		dim.resp <- dim(response)
		time <- repl
		repl <- rep(repl, dim.resp[[1]])
		counts <- rep(counts, rep(dim.resp[[2]], dim.resp[[1
			]]))
		response <- residuals.vector <- as.vector(t(response)
			)
		id <- is.na(response)
		repl <- repl[!id]
		counts <- counts[!id]
		response <- response[!id]	
	#		time <- sort(unique(repl))
		resp <- list(lower = 0, fit = 0, upper = 0, residuals
			 = repl)
		z <- qnorm(1 - (1 - coverage)/2)
		for(i in 1:length(time)) {
			yi <- response[repl == time[i]]
			countsi <- counts[repl == time[i]]
			mean.i <- sum(yi * countsi)/sum(countsi)
			variance.i <- sum(yi^2 * countsi)/sum(countsi
				) - mean.i^2
			resp$fit[i] <- mean.i
			se <- sqrt(variance.i/sum(countsi))
			resp$lower[i] <- resp$fit[i] - z * se
			resp$upper[i] <- resp$fit[i] + z * se
			if(rsd)
				resp$residuals[repl == time[i]] <- yi -
				  resp$fit[i]
		}
		if(m) {
			if(rsd) {
				residuals.vector[!id] <- resp$
				  residuals
				resp$residuals <- matrix(
				  residuals.vector, nrow = dim.resp[1
				  ], ncol = dim.resp[2], byrow = T)
			}
			temp <- smoothing(time, resp)
		}
		else {
			temp <- smoothing(time, resp$fit, control)
			if(rsd) {
				for(i in 1:length(time)) {
				  resp$residuals[repl == time[i]] <- 
				    yi - temp$fit[i]
				  residuals.vector[!id] <- resp$
				    residuals
				  temp$residuals <- matrix(
				    residuals.vector, nrow = dim.resp[
				    1], ncol = dim.resp[2], byrow = T
				    )
				}
			}
		}
		return(temp)
	}
#
# organize results to be returned
#
	compute.results <- function(mt, mf, mu, ml, mr, i, hidden, 
		profile, name)
	{
		if(hidden == 3) {
			k <- length(!is.na(mt[, 1]))
			mt[2:k,  ] <- mf[2:k,  ] <- mu[2:k,  ] <- ml[
				2:k,  ] <- NA
			dimnames(mt)[[1]][2:k] <- dimnames(mf)[[1]][2:
				k] <- dimnames(ml)[[1]][2:k] <- 
				dimnames(mu)[[1]][2:k] <- " "
			if(length(mr) > 0) {
				mr[[2:k]] <- NA
				names(mr[[2:k]]) <- " "
			}
			i <- 2
		}
		if(!any(dimnames(mt)[[1]] == name) || i == 1) {
			if(!(i == 1))
				i <- length(mt[, 1][!is.na(mt[, 1])]) +
				  1
			mt[i, 1:length(profile$time)] <- profile$time
			if(is.recursive(profile$response)) {
				mf[i, 1:length(profile$response$fit)] <-
				  profile$response$fit
				mu[i, 1:length(profile$response$upper
				  )] <- profile$response$upper
				ml[i, 1:length(profile$response$lower
				  )] <- profile$response$lower
				mr[[i]] <- profile$response$residuals
			}
			else {
				mf[i, 1:length(profile$response)] <- 
				  profile$response
				mr[[i]] <- profile$response$residuals
			}
			dimnames(mt)[[1]][i] <- dimnames(mf)[[1]][i] <- 
				dimnames(ml)[[1]][i] <- dimnames(mu)[[
				1]][i] <- name
			if(length(mr) > 0)
				names(mr)[[i]] <- name
		}
		return(list(time = mt, fit = mf, upper = mu, lower = 
			ml, residuals = mr))
	}
#
# present the results
#
	present.results <- function(results, se, rsd, call)
	{
		row <- length(results[[1]][, 1][!is.na(results[[1]][, 
			1])])
		results[[1]] <- results[[1]][1:row,  ]
		results[[2]] <- results[[2]][1:row,  ]
		results[[3]] <- results[[3]][1:row,  ]
		results[[4]] <- results[[4]][1:row,  ]
		if(rsd && length(residuals[[5]] > 0))
			results[[5]] <- results[[5]][1:row]
		else results <- results[1:4]
		if(row > 1 && all(is.na(results[[1]][2,  ]))) {
			results[[1]] <- results[[1]][1,  ]
			results[[2]] <- results[[2]][1,  ]
			results[[3]] <- results[[3]][1,  ]
			results[[4]] <- results[[4]][1,  ]
			if(rsd && length(residuals[[5]] > 0))
				results[[5]] <- results[[5]][1]
			else results <- results[1:4]
		}
		if(!se)
			if(rsd && length(residuals[[5]] > 0))
				return(list(call = call, time = 
				  results[[1]], fit = results[[2]], 
				  residuals = results[[5]]))
			else return(list(call = call, time = results[[
				  1]], fit = results[[2]]))
		else {
			if(is.vector(results[[1]]))
				return(list(call = call, time = 
				  results[[1]], fit = results[[2]], 
				  upper = results[[3]], lower = 
				  results[[4]]))
			nn <- dim(results$time)[2]
			if(all(is.na(results$upper[, 2:nn])) && all(
				is.na(results$upper[, 2:nn])))
				if(rsd && length(residuals[[5]] > 0)
				  )
				  return(list(call = call, time = 
				    results[[1]], fit = results[[2]], 
				    residuals = results[[5]]))
				else return(list(call = call, time = 
				    results[[1]], fit = results[[2]])
				    )
			else return(list(call = call, time = results[[
				  1]], fit = results[[2]], upper = 
				  results[[3]], lower = results[[4]])
				  )
		}
	}
#
# remove extraneous parentheses in formula
#
	deparen <- function(expr)
	{
		if(mode(expr) == "(")
			expr <- expr[[2]]
		if(is.recursive(expr))
			for(i in seq(along = expr))
				expr[[i]] <- Recall(expr[[i]])
		expr
	}
#
#  plot the individual profiles as an irregular multiple time series
# 
	do.plot <- function(Y, Time, counts, button, i, j, n, n.cases,
		h, v, last.profile, mean.perm, se, row, xlab, ylab, 
		xlim, ylim, smoothing, control, rsd, m, coverage, 
		name, panel = F, ...)
	{
		select.cases <- get("select.cases", inherit = T)
		mean.points <- get("mean.points", inherit = T)
		compute.results <- get("compute.results", inherit = T
			)
		id <- select.cases(h, v, i, j, n.cases, panel)
		id <- id == 3
		if(any(id)) {
			profile <- mean.points(Y[id,  ], Time, counts[
				id], smoothing, control, rsd, m, 
				coverage)
			names(profile) <- c("time", "response")
			if(se) {
				temp <- its(cbind(profile$response$
				  fit, profile$response$upper, 
				  profile$response$lower), profile$
				  time)
				color.t <- c(3, 3, 3)
				color.l <- c(1, 0, 0)
				color.m <- c(5, 0, 0)
				lty <- lty.m <- c(1, 2, 2)
			}
			else {
				temp <- its(profile$response$fit, 
				  profile$time)
				color.t <- 3
				color.l <- 1
				color.m <- c(5, 5)
				lty <- 1
				lty.m <- c(1, 1)
			}
			if(button == "Incremental scan") {
				if(!is.null(last.profile))
				  ts.lines(last.profile, col = 
				    color.l, lty = lty, type = "l")
				ts.lines(mean.perm, col = color.m, 
				  lty = lty.m, type = "l")
				ts.lines(temp, col = color.t, lty = 
				  lty, type = "l")
				last.profile <- temp
				hidden <- 2
			}
			else {
				ts.plot(mean.perm, color = color.m, 
				  lty = lty.m, type = "l", xlab = 
				  xlab, ylab = ylab, xlim = xlim, 
				  ylim = ylim, err = -1, ...)
				ts.lines(temp, col = color.t, lty = 
				  lty, type = "l")
				hidden <- 3
			}
		}
		return(list(last.profile, profile, hidden))
	}
#
# prepare the panel plot
#
	panel.plot <- function(ROW, COL, pics, mex, oldcex, cord, Y, 
		Time, counts, button, jj, n, n.cases, h, v, 
		last.profile, mean.perm, se, row, xlab, ylab, xlim, 
		ylim, var.names, etic, etic2, smoothing, control, rsd,
		m, coverage, name, panel, results, ...)
	{
#
		doaxis <- function(side, labels)
		axis(side, outer = T, pos = par("usr")[c(3, 1, 4, 2)[
			side]], labels = labels)	#
		domtext <- function(label, side, line, srt)
		mtext(label, side, line, outer = T, at = 0.5, srt = 
			srt)	#
		do.plot <- get("do.plot", inherit = T)
		compute.results <- get("compute.results", inherit = T
			)
		mar <- 0.25	
	#	par(mfrow = c(ROW, COL), tck = -0.03/max(ROW, COL))
		par(cex = oldcex * max(7.7/(2 * max(ROW, COL) + 3), 
			0.6))
		par(oma = c(4.3 - mar, 4.3, 4.3, 1.5)/mex, mar = c(
			mar, mar, mar, mar)/mex)
		par(new = F)
		plot(cord[1:2], cord[3:4], xlab = "", ylab = "", axes
			 = F, type = "n", err = -1)
		cr <- 1
		for(i in 1:ROW)
			for(j in 1:COL) {
				if(cr > pics)
				  next
				par(mfg = c(i, j, ROW, COL))
				output <- do.plot(Y, Time, counts, 
				  button, cr, jj, n, n.cases, h, v, 
				  last.profile, mean.perm, se, row, 
				  xlab, ylab, xlim, ylim, smoothing, 
				  control, rsd, m, coverage, name, 
				  panel, axes = F, ...)
				par(mex = par("cex") * 1.5)
				mtext(paste(var.names[3], ": ", etic[
				  cr], collapse = ""), side = 3, line
				   = -1, outer = F)
				box()
				if(i == ROW || (cr >= pics - COL + 1)
				  )
				  doaxis(1, T)
				if(j == 1)
				  doaxis(2, T)
				last.profile <- output[[1]]
				if(!is.null(jj))
				  name <- paste(etic[cr], ",", etic2[
				    jj], collapse = "")
				else name <- paste(etic[cr], collapse
				     = "")
				results <- compute.results(results[[1
				  ]], results[[2]], results[[3]], 
				  results[[4]], results[[5]], row, 2, 
				  output[[2]], name)
				cr <- cr + 1
				row <- row + 1
			}
		par(mex = mex)
		par(fig = c(0, 1, 0, 1), usr = c(0, 1, 0, 1), mar = 
			rep(0, 4), cex = 1.2 * oldcex)
		domtext(xlab[1], 1, 3 - mar, 0)
		domtext(ylab, 2, 3, 90)
		return(results)
	}
#
# create dialog object
#
	dialog.create <- function(panel, onegiven, var.names, etic, 
		etic2, level.first, level.second, level.mode, height
		)
	{
		height <- 10 * height
		first.ctl <- listbox.control(label = var.names[3], 
			listvalues = etic, selected = level.first, 
			position = c(6, 15), size = c(60, height))
		if(onegiven) {
			mode.ctl <- listbox.control(label = "Mode", 
				listvalues = c("Single scan", 
				"Incremental scan"), selected = 
				level.mode, position = c(6, 15 + 
				height + 20), size = c(60, 20))
			obj.dialog <- dialogbox(title = 
				"Conditioning levels", controls = 
				list(first.ctl, mode.ctl), buttons = 
				c("Ok", "Quit"), default.button = c(
				"Ok"))
		}
		else {
			if(panel) {
				second.ctl <- listbox.control(label
				   = var.names[4], listvalues = etic2,
				  selected = level.second, position
				   = c(6, 15), size = c(60, height))	
	#			mode.ctl <- listbox.control(label="Mode", 
#			listvalues=c("Single scan","Incremental scan"), 
#			selected = level.mode, 
#			position=c(6,15+height+20), 
#			size=c(60,20))
				obj.dialog <- dialogbox(title = 
				  "Conditioning levels", controls = 
				  list(second.ctl), buttons = c("Ok", 
				  "Quit"), default.button = c("Ok"))
			}
			else {
				second.ctl <- listbox.control(label
				   = var.names[4], listvalues = etic2,
				  selected = level.second, position
				   = c(6, 15 + height + 20), size = c(
				  60, height))
				mode.ctl <- listbox.control(label = 
				  "Mode", listvalues = c(
				  "Single scan", "Incremental scan"), 
				  selected = level.mode, position = c(
				  6, 15 + 2 * height + 40), size = c(
				  60, 20))
				obj.dialog <- dialogbox(title = 
				  "Conditioning levels", controls = 
				  list(first.ctl, second.ctl, 
				  mode.ctl), buttons = c("Ok", "Quit"
				  ), default.button = c("Ok"))	
	#		dialogbox.set.control.name(obj.dialog,1,"First variable")
			}
		}
		return(obj.dialog)
	}
#
###################################################################################################
	call <- match.call()
	one.group.only <- nogiven <- F
	m <- F
	formula <- deparen(formula)
	if(mode(formula) != "call")
		stop("formula is specified incorrectly")
	else {
		if(!is.recursive(formula[[3]]))
			nogiven <- T
		else {
			if(formula[[3]][[1]] != as.name("|"))
				stop("malformed conditioning formula"
				  )
			formula[[3]][[1]] <- as.name("+")
		}
		expr <- attr(terms(formula), "variables")
		labels <- as.character(expr)
		var.names <- all.vars(expr)
		count <- length(expr)
		if(count > 4)
			stop("At most 2 given covariates allowed in rm.mean"
				)
	}
	onegiven <- count == 3
	if(!is.list(data))
		stop("data must be in a list")
	if(is.null(names(data)))
		stop("elements in data must have a name")
	if(rsd && !missing(smoothing)) {
		cat("\nSorry, residuals option not available with a smoother\n"
			)
		rsd <- F
	}
#
# select subset if necessary
#
	if(!missing(subset)) {
		id <- eval(substitute(subset), data)
		if(any(id)) {
			for(i in 1:length(data))
				if(is.matrix(data[[i]])) data[[i]] <- 
				    data[[i]][id,  ] else if(names(
				  data)[[i]] != var.names[2])
				  data[[i]] <- data[[i]][id]
		}
		else stop("subset condition not satisfied by any case"
				)
	}
#
# evaluate expression
#
	var.names <- all.vars(expr)
	Y <- eval(expr[[1]], data)
	n.cases <- dim(Y)[1]
	Time <- eval(expr[[2]], data)
	if(is.matrix(Time)) {
		Y <- resp.union(Y, Time)
		Time <- munion(Time)
	}
	if(!is.numeric(Y) || !is.numeric(Time)) stop(
			"response && time must be numeric")	#
# set up the smoother
#
	if(missing(counts)) {
		if(is.na(match("counts", names(data))))
			counts <- data$counts <- rep(1, n.cases)
		else counts <- data$counts
	}
	if(missing(smoothing)) {
		m <- T
		smoothing <- function(x, y)
		return(list(x, y))
	}
	else if(smoothing == "loess") {
		if(is.null(control))
			control <- loess.ctrl(...)
		smoothing <- function(x, y, control)
		{
			rm.loess(x, y, span = control$span, degree = 
				control$degree, family = control$
				family, evaluation = control$
				evaluation, coverage = control$
				coverage)
		}
	}
	else if(smoothing == "ksmooth") {
		if(is.null(control))
			control <- ksmooth.ctrl(...)
		smoothing <- function(x, y, control)
		{
			temp <- ksmooth(x, y, kernel = control$kernel,
				bandwidth = control$bandwidth, 
				n.points = control$n.points)
			return(list(time = temp[[1]], list(fit = temp[[
				2]])))
		}
	}
	mean.profile <- mean.points(Y, Time, counts, smoothing, 
		control, rsd, m, coverage)
	names(mean.profile) <- c("time", "response")
	if(se)
		mean.perm <- its(cbind(mean.profile$response$fit, 
			mean.profile$response$upper, mean.profile$
			response$lower), mean.profile$time)
	else mean.perm <- its(cbind(mean.profile$response$fit, 	
	# added to overcome Splus color bug in ts.plot
		mean.profile$response$fit), mean.profile$time)
	if(!is.null(batch)) {
		n.conditions <- length(batch)
		mt <- mf <- mu <- ml <- matrix(NA, nrow = 
			n.conditions + 1, ncol = length(mean.profile$
			time), dimnames = list(rep(" ", n.conditions + 
			1), NULL))
		mr <- list()
		results <- compute.results(mt, mf, mu, ml, mr, 1, 2, 
			mean.profile, "grandmean")
		if(n.conditions == 0)
			return(present.results(results, se, rsd, call
				))
		for(i in 1:n.conditions) {
			condition <- batch[[i]]
			id <- eval(substitute(condition), data)
			if(any(id)) {
				data.temp <- data
				for(j in 1:length(data)) {
				  if(is.matrix(data[[j]]))
				    data[[j]] <- data[[j]][id,  ]
				  else if(names(data)[[j]] != 
				    var.names[2])
				    data[[j]] <- data[[j]][id]
				}
				Y <- eval(expr[[1]], data)
				Time <- eval(expr[[2]], data)
				if(is.matrix(Time)) {
				  Y <- resp.union(Y, Time)
				  Time <- munion(Time)
				}
				profile <- mean.points(Y, Time, 
				  counts[id], smoothing, control, rsd,
				  m, coverage)
				names(profile) <- c("time", 
				  "response")
				results <- compute.results(results[[1
				  ]], results[[2]], results[[3]], 
				  results[[4]], results[[5]], 2, 2, 
				  profile, paste(batch[[i]], collapse
				   = ""))
				data <- data.temp
			}
			else stop(paste("component", i, 
				  "of the batch condition not satisfied by any case",
				  collapse = ""))
		}
# invisible()	
		return(present.results(results, se, rsd, call))
	}
#
# prepare the graphical parameters
#
	if(missing(ylab))
		ylab <- paste(expr[[1]], collapse = " ")
	if(missing(timelab))
		xlab <- paste(expr[[2]], collapse = " ")
	else if(!is.character(timelab))
		stop("timelab must be a character vector")
	else xlab <- timelab
	if(missing(ylim))
		ylim <- range(Y[!is.na(Y)])
	if(missing(timelim))
		xlim <- range(Time[!is.na(Time)])
	else xlim <- timelim
	oldpar <- par()
	on.exit(par(oldpar))
	par(err = -1)
	frame()	#
# no conditioning variables case first
#
	if(nogiven) {
		if(se) {
			color <- c(5, 5, 5)
			lty <- c(1, 2, 2)
		}
		else {
			color <- 5
			lty <- 1
		}
		ts.plot(mean.perm, color = color, type = "l", lty = 
			lty, xlab = xlab, ylab = ylab, xlim = xlim, 
			ylim = ylim, err = -1, ...)
		mt <- mf <- mu <- ml <- matrix(NA, nrow = 1, ncol = 
			length(mean.profile$time), dimnames = list(c(
			" "), NULL))
		mr <- list()
		results <- compute.results(mt, mf, mu, ml, mr, 1, 2, 
			mean.profile, "grandmean")
		invisible()
		return(present.results(results, se, rsd, call))
	}
#
# set up levels for conditioning variables
#
	H <- eval(expr[[3]], data)
	if(!onegiven)
		V <- eval(expr[[4]], data)
	if(!onegiven && length(H) != length(V))
		stop("different number of cases between covariates")
	if(!missing(given.values)) {
		if(!is.list(given.values))
			given.values <- list(given.values)
		do.given <- function(G, given)
		{
			lev <- c()
			if(is.factor(G)) {
				G <- category(G)
				lev <- levels(given)
				index <- match(lev, levels(G))
				int <- matrix(rep(sort(unique(index)),
				  2), ncol = 2)
			}
			else int <- if(is.matrix(given) && dim(given)[
				  2] == 2) given else cbind(given, 
				    given)
			for(i in 1:dim(int)[1])
				lev[i] <- paste(round(int[i,  ], 2), 
				  collapse = " to ")
			list(given = G, levels = lev, intervals = int
				)
		}
	}
	else {
		given.values <- list()
		do.given <- function(G, dummy)
		{
			quart.intervals <- function(x)
			{
				temp <- summary(x)
				temp <- matrix(c(temp[1:3], temp[5], 
				  temp[2:3], temp[5:6]), ncol = 2)
				temp[1, 1] <- temp[1, 1] - temp[1, 1]/
				  100
				temp[4, 2] <- temp[4, 2] + temp[4, 2]/
				  100
				return(temp)
			}
			lev <- c()
			if(is.factor(G)) {
				G <- category(G)
				lev <- levels(G)
				int <- matrix(rep(sort(unique(G)), 2),
				  ncol = 2)
			}
			else {
				int <- quart.intervals(G)
				for(i in 1:dim(int)[1])
				  lev[i] <- paste(round(int[i,  ], 2),
				    collapse = " to ")
			}
			list(given = G, levels = lev, intervals = int
				)
		}
	}
	h <- do.given(H, given.values[[1]])
	n <- height <- length(h$levels)
	etic <- rep(" ", n)
	for(i in 1:n)
		etic[i] <- paste(h$levels[i], collapse = "")
	if(n > 8)
		height <- 8
	if(!onegiven) {
		v <- do.given(V, given.values[[2]])
		n2 <- length(v$levels)
		nn <- n * n2 + 1
		j.v <- 1
		etic2 <- rep("", n2)
		for(i in 1:n2)
			etic2[i] <- paste(v$levels[i], collapse = "")
	}
	else {
		v <- j.v <- NULL
		nn <- n + 1
	}
	if(onegiven & n == 1)
		one.group.only <- T
	if(!onegiven) {
		if(n == 1 & n2 > 1) {
			n <- n2
			nn <- n + 1
			h <- v
			etic <- etic2
			onegiven <- T
			cat("Warning message: only one possible level for the second variable in formula \n\n\t\t\tGraphical interaction dismissed for second variable"
				)
		}
		else if(n > 1 & n2 == 1)
			v <- j.v <- NULL
		else if(n == 1 & n2 == 1)
			one.group.only <- T
	}
#
# identify the first two profile to be plotted	
#
	indx <- select.cases(h, v, 1, j.v, n.cases)
	indx <- indx == 3
	if(any(indx)) {
#	last.profile <- 
		l.p <- mean.points(Y[indx,  ], Time, counts[indx], 
			smoothing, control, rsd, m, coverage)	
	#	names(last.profile) <- 
		names(l.p) <- c("time", "response")
		if(!onegiven)
			name <- paste(etic[1], ",", etic2[1], 
				collapse = "")
		else name <- etic[1]
	}
# else last.profile <- NULL
	last.profile <- NULL
	if(se) {
		color.m <- c(5, 5, 5)
		lty <- lty.m <- c(1, 2, 2)
		color.t <- c(3, 3, 3)	#	if(!is.null(last.profile)){
#	last.profile <- its(cbind(last.profile$response$fit, 
#			last.profile$response$upper,last.profile$response$lower), last.profile$time)
#	color.t <- c(3,3,3)
#	}
		l.p <- its(cbind(l.p$response$fit, l.p$response$upper,
			l.p$response$lower), l.p$time)
		color.t <- c(3, 3, 3)
	}
	else {
		color.m <- c(5, 5)
		color.t <- 3
		lty.m <- c(1, 1)
		lty <- 1	#	if(!is.null(last.profile)){
#	last.profile <- its(last.profile$response$fit, last.profile$time)
#	color.t <- 3
#	}
		l.p <- its(l.p$response$fit, l.p$time)
		color.t <- 3
	}
	if(!panel) {
		ts.plot(mean.perm, color = color.m, lty = lty.m, type
			 = "l", xlab = xlab, ylab = ylab, xlim = xlim,
			ylim = ylim, err = -1, ...)
		ts.lines(l.p, col = color.t, type = "l", lty = lty)	
	###############
#if(!is.null(last.profile))
#	ts.lines(last.profile,col=color.t,type="l",lty=lty)
##############
	}
##################
	mt <- mf <- mu <- ml <- matrix(NA, nrow = nn, ncol = length(
		mean.profile$time), dimnames = list(rep(" ", nn), 
		NULL))
	mr <- list()
	results <- compute.results(mt, mf, mu, ml, mr, 1, 2, 
		mean.profile, "grandmean")
	if(one.group.only)
		closing.message()
	row <- 2
	if(!is.null(last.profile) & !panel) results <- 
			compute.results(results[[1]], results[[2]], 
			results[[3]], results[[4]], results[[5]], row,
			2, l.p, name)	#
# deal with the panel option
#
	if(panel) {
		pics <- length(h$intervals[, 1])
		ROW <- ceiling(round(sqrt(pics)))
		COL <- ceiling(pics/ROW)
		if(ROW * COL < pics)
			stop("rows or columns too small for given values"
				)
		oldcex <- par("cex")
		mex <- par("mex")
		oldpar <- par("omi", "mar", "cex", "tck", "mfg", 
			"mgp", "pty", "mfrow", "err", "fig", "fty")
		on.exit({
			par(oldpar)
			par(usr = cord, new = F)
		}
		)
		ylim[2] <- ylim[2] + 0.1 * (ylim[2])
		cord <- c(xlim, ylim)
		if(onegiven) {
			results <- panel.plot(ROW, COL, pics, mex, 
				oldcex, cord, Y, Time, counts, 
				"Single scan", NULL, n, n.cases, h, 
				NULL, last.profile, mean.perm, se, 
				row, xlab, ylab, xlim, ylim, 
				var.names, etic, etic2, smoothing, 
				control, rsd, m, coverage, name, 
				panel, results, ...)	
	#		dialog.destroy(obj.dialog)
#		dialog.off()
#		invisible(present.results(results, se, rsd,call))
		}
		else {
			par(oldpar)
			par(usr = cord, new = F)
			cat("Warning: Incremental scan of profiles not\navailable with two covariates in panel plots\n\n"
				)
			message.interactive()
			results <- panel.plot(ROW, COL, pics, mex, 
				oldcex, cord, Y, Time, counts, 
				"Single scan", 1, n, n.cases, h, v, 
				last.profile, mean.perm, se, row, 
				xlab, ylab, xlim, ylim, var.names, 
				etic, etic2, smoothing, control, rsd, 
				m, coverage, name, panel, results, 
				...)
			par(oldpar)
			par(usr = cord, new = F)
			level.first <- level.second <- level.mode <- 
				1
			repeat {
				obj.dialog <- dialog.create(panel, 
				  onegiven, var.names, etic, etic2, 
				  level.first, level.second, 
				  level.mode, height)
				buttons <- dialogbox.display(
				  obj.dialog)
				if(buttons$button == "Quit") {
				  break
				}
				else {
				  values <- buttons$values
				  nv <- length(values)
				  j <- (1:nv)[names(values) == 
				    var.names[4]]
				  level.second <- (1:n2)[etic2 == 
				    values[[j]]]
				  mode <- values$Mode	
	#				if (mode == "Single scan") level.mode <- 1
#				else level.mode <- 2
				  par(oldpar)
				  par(usr = cord, new = F)
				  row <- dim(results[[1]])[1]
				  results <- panel.plot(ROW, COL, 
				    pics, mex, oldcex, cord, Y, Time, 
				    counts, "Single scan", 
				    level.second, n, n.cases, h, v, 
				    last.profile, mean.perm, se, row, 
				    xlab, ylab, xlim, ylim, var.names,
				    etic, etic2, smoothing, control, 
				    rsd, m, coverage, name, panel, 
				    results, ...)
				  par(oldpar)
				  par(usr = cord, new = F)
				}
			}
		}
	}
	else {
#
# interactive graphics
#
		message.interactive()
		level.first <- level.second <- level.mode <- 1
		repeat {
			obj.dialog <- dialog.create(panel, onegiven, 
				var.names, etic, etic2, level.first, 
				level.second, level.mode, height)
			buttons <- dialogbox.display(obj.dialog)
			if(buttons$button == "Quit") {
				break
			}
			else {
				values <- buttons$values
				nv <- length(values)
				i <- (1:nv)[names(values) == 
				  var.names[3]]
				mode <- values$Mode
				if(mode == "Single scan")
				  level.mode <- 1
				else level.mode <- 2
				if(!onegiven) {
				  j <- (1:nv)[names(values) == 
				    var.names[4]]
				  level.first <- (1:n)[etic == values[[
				    i]]]
				  level.second <- (1:n2)[etic2 == 
				    values[[j]]]
				  name <- paste(etic[i], ",", etic2[j
				    ], collapse = "")
				}
				else {
				  level.first <- (1:n)[etic == values[[
				    i]]]
				  level.second <- NULL
				  name <- paste(etic[i], collapse = 
				    "")
				}
				output <- do.plot(Y, Time, counts, 
				  values$Mode, level.first, 
				  level.second, n, n.cases, h, v, 
				  last.profile, mean.perm, se, row, 
				  xlab, ylab, xlim, ylim, smoothing, 
				  control, rsd, m, coverage, name, 
				  ...)
				last.profile <- output[[1]]
				results <- compute.results(results[[1
				  ]], results[[2]], results[[3]], 
				  results[[4]], results[[5]], row, 
				  output[[3]], output[[2]], name)
			}
		}
	}
	par(err = 0)
	invisible(present.results(results, se, rsd, call))
}
"rm.plot"<-
function(formula, data, subset, given.values, timelab, ylab, timelim, 
	ylim, panel = T, ...)
{
##############################################################################
#	      	  *** DEFINITION OF INTERNAL FUNCTIONS ***		     #
##############################################################################
#
# starting interaction message
#
	message.interactive <- function()
	cat("Control passing to `Conditioning levels` menu.\nClick on 'OK' after selection, `Quit` to stop.\n"
		)	#
# closing message
#
	closing.message <- function()
	{
		cat("Warning message:\nOne group of profiles only found to be usable.\nGraphical interaction disactivated.\n"
			)
		invisible()
	}
#	
# create the sorted union of the elements in a matrix 
#
	munion <- function(matrix)
	sort(unique(as.vector(matrix)))	#
# create a multiple irregular time series 
#
	resp.union <- function(Y, Time)
	{
		munion <- get("munion", inherit = T)
		Time.big <- munion(Time)
		n <- dim(Y)[[1]]
		l <- length(Time.big)
		Y.big <- matrix(rep(NA, n * l), n, l)
		for(i in 1:n)
			Y.big[i,  ][!is.na(match(Time.big, Time[i,  ]
				))] <- Y[i,  ]
		return(Y.big)
	}
#	
# find cases corresponding to levels of two covariates
#
	select.cases <- function(longer, shorter, which.levels.longer,
		which.levels.shorter, n.cases, panel = F)
	{
		pick <- function(G, g, level)
		G >= g[level, 1] & G <= g[level, 2] & !is.na(G)
		if(panel)
			id <- rep(0, n.cases)
		else id <- rep(1, n.cases)
		if(is.null(shorter))
			id[pick(longer$given, longer$intervals, 
				which.levels.longer)] <- 3
		else {
			temp <- temp1 <- rep(1, n.cases)
			temp[pick(longer$given, longer$intervals, 
				which.levels.longer)] <- 3
			temp1[pick(shorter$given, shorter$intervals, 
				which.levels.shorter)] <- 3
			temp3 <- (temp == 3) & (temp1 == 3)
			if(panel) {
				id[temp == 3] <- 1
				id[temp3] <- 3
			}
			else id[temp3] <- 3
		}
		return(id)
	}
# 
# remove extraneous parentheses in parse tree
#
	deparen <- function(expr)
	{
		if(mode(expr) == "(")
			expr <- expr[[2]]
		if(is.recursive(expr))
			for(i in seq(along = expr))
				expr[[i]] <- Recall(expr[[i]])
		expr
	}
#
#  plot the individual profiles as an irregular multiple time series
# 
	do.plot <- function(Y, Time, button, i, j, hidden, n, n.cases,
		h, v, xlab, ylab, xlim, ylim, panel = F, ...)
	{
		Y <- matrix(Y, nrow = n.cases)
		select.cases <- get("select.cases", inherit = T)
		if(button == "Show unselected")
			cases.selected <- rep(1, n.cases)
		else cases.selected <- rep(0, n.cases)
		id <- select.cases(h, v, i, j, n.cases, panel)
		if(panel) {
			cases.selected <- id
			if(button == "Hide unselected")
				cases.selected[cases.selected == 1] <- 
				  0
			id <- (cases.selected != 0)
			cases.selected <- cases.selected[id]
			n.cases <- length(id[id == T])
			Y <- matrix(Y[id,  ], nrow = n.cases)
			lty <- rep(1, n.cases)
		}
		else {
			cases.selected[id == 3] <- 3
			lty <- rep(1, n.cases)
		}
		par(new = T)
		x <- order(cases.selected)
		if(n.cases == 1) {
# added to overcome Splus color bug in ts.plot
			series <- matrix(rep(Y[x,  ], 2), nrow = 2, 
				byrow = T)
			color <- rep(cases.selected[x], 2)
		}
		else {
			series <- matrix(Y[x,  ], nrow = n.cases)
			color <- cases.selected[x]
		}
		ts.plot(its(t(series), Time), color = color, type = 
			"l", lty = lty, xlab = xlab, ylab = ylab, 
			xlim = xlim, ylim = ylim, err = -1, ...)
	}
#
# create panel plot
#
	panel.plot <- function(ROW, COL, pics, mex, oldcex, cord, Y, 
		Time, button, hidden, n, n.cases, h, v, jj, var.names,
		etic, xlab, ylab, xlim, ylim, ...)
	{
#
		doaxis <- function(side, labels)
		axis(side, outer = T, pos = par("usr")[c(3, 1, 4, 2)[
			side]], labels = labels)	#
		domtext <- function(label, side, line, srt)
		mtext(label, side, line, outer = T, at = 0.5, srt = 
			srt)	#
		do.plot <- get("do.plot", inherit = T)
		mar <- 0.25	
	#	par(mfrow = c(ROW, COL), tck = -0.03/max(ROW, COL))
		par(cex = oldcex * max(7.7/(2 * max(ROW, COL) + 3), 
			0.6))
		par(oma = c(4.3 - mar, 4.3, 4.3, 1.5)/mex, mar = c(
			mar, mar, mar, mar)/mex)
		par(new = F)
		plot(cord[1:2], cord[3:4], xlab = "", ylab = "", axes
			 = F, type = "n", err = -1)
		cr <- 1
		for(i in 1:ROW)
			for(j in 1:COL) {
				if(cr > pics)
				  next
				par(mfg = c(i, j, ROW, COL))
				do.plot(Y, Time, button, cr, jj, 
				  hidden, n, n.cases, h, v, xlab, 
				  ylab, xlim, ylim, panel = T, axes
				   = F, ...)
				par(mex = par("cex") * 1.5)
				mtext(paste(var.names[3], ": ", etic[
				  cr], collapse = ""), side = 3, line
				   = -1, outer = F)
				box()
				if(i == ROW || (cr >= pics - COL + 1)
				  )
				  doaxis(1, T)
				if(j == 1)
				  doaxis(2, T)
				cr <- cr + 1
			}
		par(mex = mex)
		par(fig = c(0, 1, 0, 1), usr = c(0, 1, 0, 1), mar = 
			rep(0, 4), cex = 1.2 * oldcex)
		domtext(xlab[1], 1, 3 - mar, 0)
		domtext(ylab, 2, 3, 90)
	}
#
# create dialog object
#
	dialog.create <- function(panel, onegiven, var.names, etic, 
		etic2, level.first, level.second, level.mode, height
		)
	{
		height <- 10 * height
		first.ctl <- listbox.control(label = var.names[3], 
			listvalues = etic, selected = level.first, 
			position = c(6, 15), size = c(60, height))
		if(onegiven) {
			mode.ctl <- listbox.control(label = "Mode", 
				listvalues = c("Show unselected", 
				"Hide unselected"), selected = 
				level.mode, position = c(6, 15 + 
				height + 20), size = c(60, 20))
			obj.dialog <- dialogbox(title = 
				"Conditioning levels", controls = 
				list(first.ctl, mode.ctl), buttons = 
				c("Ok", "Quit"), default.button = c(
				"Ok"))
		}
		else {
			if(panel) {
				second.ctl <- listbox.control(label
				   = var.names[4], listvalues = etic2,
				  selected = level.second, position
				   = c(6, 15), size = c(60, height))
				mode.ctl <- listbox.control(label = 
				  "Mode", listvalues = c(
				  "Show unselected", 
				  "Hide unselected"), selected = 
				  level.mode, position = c(6, 15 + 
				  height + 20), size = c(60, 20))
				obj.dialog <- dialogbox(title = 
				  "Conditioning levels", controls = 
				  list(second.ctl, mode.ctl), buttons
				   = c("Ok", "Quit"), default.button
				   = c("Ok"))
			}
			else {
				second.ctl <- listbox.control(label
				   = var.names[4], listvalues = etic2,
				  selected = level.second, position
				   = c(6, 15 + height + 20), size = c(
				  60, height))
				mode.ctl <- listbox.control(label = 
				  "Mode", listvalues = c(
				  "Show unselected", 
				  "Hide unselected"), selected = 
				  level.mode, position = c(6, 15 + 2 * 
				  height + 40), size = c(60, 20))
				obj.dialog <- dialogbox(title = 
				  "Conditioning levels", controls = 
				  list(first.ctl, second.ctl, 
				  mode.ctl), buttons = c("Ok", "Quit"
				  ), default.button = c("Ok"))	
	#		dialogbox.set.control.name(obj.dialog,1,"First variable")
			}
		}
		return(obj.dialog)
	}
#
##########################################################################
#			*** MAIN PROGRAM ***				 #
##########################################################################
	one.group.only <- nogiven <- F
	formula <- deparen(formula)
	if(mode(formula) != "call")
		stop("formula is specified incorrectly")
	else {
		if(!is.recursive(formula[[3]]))
			nogiven <- T
		else {
			if(formula[[3]][[1]] != as.name("|"))
				stop("malformed conditioning formula"
				  )
			formula[[3]][[1]] <- as.name("+")
		}
		expr <- attr(terms(formula), "variables")
		labels <- as.character(expr)
		var.names <- all.vars(expr)
		count <- length(expr)
		if(count > 4)
			stop("At most 2 given covariates allowed in rm.mean"
				)
	}
	onegiven <- count == 3
	if(!is.list(data))
		stop("data must be in a list")
	if(is.null(names(data))) stop(
			"elements in data must have a name")	#
# select subset if necessary
#
	if(!missing(subset)) {
		id <- eval(substitute(subset), data)
		if(any(id)) {
			for(i in 1:length(data))
				if(is.matrix(data[[i]])) data[[i]] <- 
				    data[[i]][id,  ] else if(names(
				  data)[[i]] != var.names[2])
				  data[[i]] <- data[[i]][id]
		}
		else stop("subset condition not satisfied by any case"
				)
	}
#
# evaluate expression
#
	Y <- eval(expr[[1]], data)
	n.cases <- dim(Y)[1]
	Time <- eval(expr[[2]], data)
	if(is.matrix(Time)) {
		Y <- resp.union(Y, Time)
		Time <- munion(Time)
	}
	if(!is.numeric(Y) || !is.numeric(Time)) stop(
			"response && time must be numeric")	#
# prepare the graphical parameters
#
	if(missing(ylab))
		ylab <- paste(expr[[1]], collapse = " ")
	if(missing(timelab))
		xlab <- paste(expr[[2]], collapse = " ")
	else if(!is.character(timelab))
		stop("timelab must be a character vector")
	else xlab <- timelab
	if(missing(ylim))
		ylim <- range(Y[!is.na(Y)])
	if(missing(timelim))
		xlim <- range(Time[!is.na(Time)])
	else xlim <- timelim
	frame()
	oldpar <- par()
	on.exit(par(oldpar))
	par(err = -1)
	if(nogiven) {
		ts.plot(its(t(Y), Time), col = rep(1, n.cases), type
			 = "l", lty = rep(1, n.cases), xlab = xlab, 
			ylab = ylab, xlim = xlim, ylim = ylim, err = 
			-1
			, ...)
		invisible()
		return()
	}
	H <- eval(expr[[3]], data)
	if(!onegiven)
		V <- eval(expr[[4]], data)
	if(!onegiven && length(H) != length(V))
		stop("different number of cases between covariates")
	if(!missing(given.values)) {
		if(!is.list(given.values))
			given.values <- list(given.values)
		do.given <- function(G, given)
		{
			lev <- c()
			if(is.factor(G)) {
				G <- category(G)
				lev <- levels(given)
				index <- match(lev, levels(G))
				int <- matrix(rep(sort(unique(index)),
				  2), ncol = 2)
			}
			else int <- if(is.matrix(given) && dim(given)[
				  2] == 2) given else cbind(given, 
				    given)
			for(i in 1:dim(int)[1])
				lev[i] <- paste(round(int[i,  ], 2), 
				  collapse = " to ")
			list(given = G, levels = lev, intervals = int
				)
		}
	}
	else {
		given.values <- list()
		do.given <- function(G, dummy)
		{
			quart.intervals <- function(x)
			{
				temp <- summary(x)
				temp <- matrix(c(temp[1:3], temp[5], 
				  temp[2:3], temp[5:6]), ncol = 2)
				temp[1, 1] <- temp[1, 1] - temp[1, 1]/
				  100
				temp[4, 2] <- temp[4, 2] + temp[4, 2]/
				  100
				return(temp)
			}
			lev <- c()
			if(is.factor(G)) {
				G <- category(G)
				lev <- levels(G)
				int <- matrix(rep(sort(unique(G)), 2),
				  ncol = 2)
			}
			else {
				int <- quart.intervals(G)
				for(i in 1:dim(int)[1])
				  lev[i] <- paste(round(int[i,  ], 2),
				    collapse = " to ")
			}
			list(given = G, levels = lev, intervals = int
				)
		}
	}
	h <- do.given(H, given.values[[1]])
	if(!onegiven)
		v <- do.given(V, given.values[[2]])
	else v <- NULL
	cov.lab <- var.names[3:count]	
	###################################################################################
	h <- do.given(H, given.values[[1]])
	n <- height <- length(h$levels)
	if(n > 8)
		height <- 8
	etic <- rep(" ", n)
	for(i in 1:n)
		etic[i] <- paste(h$levels[i], collapse = "")
	if(!onegiven) {
		v <- do.given(V, given.values[[2]])
		n2 <- length(v$levels)
		if(n2 < height)
			height <- n2
		nn <- n * n2 + 1
		j.v <- 1
		etic2 <- rep(" ", n2)
		for(i in 1:n2)
			etic2[i] <- paste(v$levels[i], collapse = "")
	}
	else {
		v <- j.v <- NULL
		nn <- n + 1
	}
	if(onegiven & n == 1)
		one.group.only <- T
	if(!onegiven) {
		if(n == 1 & n2 > 1) {
			n <- n2
			nn <- n + 1
			h <- v
			etic <- etic2
			onegiven <- T
			cat("Warning message: only one possible level for the second variable in formula \n\n\t\t\tGraphical interaction dismissed for second variable"
				)
		}
		else if(n > 1 & n2 == 1)
			v <- j.v <- NULL
		else if(n == 1 & n2 == 1)
			one.group.only <- T
	}
# interactive graphics	
	hidden <- 2	# not selected
	if(!panel) {
		cases.selected <- rep(1, n.cases)
		indx <- select.cases(h, NULL, 1, NULL, n.cases)
		cases.selected[indx == 3] <- 3
		x <- order(cases.selected)
		temp <- its(t(Y[x,  ]), Time)
		ts.plot(temp, color = cases.selected[x], type = "l", 
			lty = rep(1, n.cases), xlab = xlab, ylab = 
			ylab, xlim = xlim, ylim = ylim, err = -1, ...
			)
		if(one.group.only)
			closing.message()
	}
#
# deal with the panel option
#
	if(panel) {
		pics <- length(h$intervals[, 1])
		ROW <- ceiling(round(sqrt(pics)))
		COL <- ceiling(pics/ROW)
		if(ROW * COL < pics)
			stop("rows or columns too small for given values"
				)
		oldcex <- par("cex")
		mex <- par("mex")
		oldpar <- par("omi", "mar", "cex", "tck", "mfg", 
			"mgp", "pty", "mfrow", "err", "fig", "fty")
		on.exit({
			par(oldpar)
			par(usr = cord, new = F)
		}
		)
		ylim[2] <- ylim[2] + 0.1 * (ylim[2])
		cord <- c(xlim, ylim)
		if(onegiven) {
			panel.plot(ROW, COL, pics, mex, oldcex, cord, 
				Y, Time, "Hide unselected", hidden, n,
				n.cases, h, NULL, NULL, var.names, 
				etic, xlab, ylab, xlim, ylim, ...)
			invisible()
		}
		else {
			par(oldpar)
			par(usr = cord, new = F)	
	#		panel.par(ROW,COL,oldcex,mex,cord)
			panel.plot(ROW, COL, pics, mex, oldcex, cord, 
				Y, Time, "Show unselected", hidden, n,
				n.cases, h, v, 1, var.names, etic, 
				xlab, ylab, xlim, ylim, ...)
			message.interactive()
			level.first <- level.second <- level.mode <- 
				1
			repeat {
				obj.dialog <- dialog.create(panel, 
				  onegiven, var.names, etic, etic2, 
				  level.first, level.second, 
				  level.mode, height)
				buttons <- dialogbox.display(
				  obj.dialog)
				if(buttons$button == "Quit") {
				  break
				}
				else {
				  values <- buttons$values
				  nv <- length(values)
				  j <- (1:nv)[names(values) == 
				    var.names[4]]
				  level.second <- (1:n2)[etic2 == 
				    values[[j]]]
				  mode <- buttons$values$Mode
				  if(mode == "Show unselected")
				    level.mode <- 1
				  else level.mode <- 2
				  par(oldpar)
				  par(usr = cord, new = F)
				  panel.plot(ROW, COL, pics, mex, 
				    oldcex, cord, Y, Time, mode, 
				    hidden, n, n.cases, h, v, 
				    level.second, var.names, etic, 
				    xlab, ylab, xlim, ylim, ...)
				}
			}
		}
	}
	else {
		message.interactive()
		level.first <- level.second <- level.mode <- 1
		repeat {
			obj.dialog <- dialog.create(panel, onegiven, 
				var.names, etic, etic2, level.first, 
				level.second, level.mode, height)
			buttons <- dialogbox.display(obj.dialog)
			if(buttons$button == "Quit") {
				break
			}
			else {
				values <- buttons$values
				nv <- length(values)
				i <- (1:nv)[names(values) == 
				  var.names[3]]
				mode <- buttons$values$Mode
				if(mode == "Show unselected")
				  level.mode <- 1
				else level.mode <- 2
				if(!onegiven) {
				  j <- (1:nv)[names(values) == 
				    var.names[4]]
				  level.first <- (1:n)[etic == values[[
				    i]]]
				  level.second <- (1:n2)[etic2 == 
				    values[[j]]]
				}
				else {
				  level.first <- (1:n)[etic == values[[
				    i]]]
				  level.second <- NULL
				}
				do.plot(Y, Time, mode, level.first, 
				  level.second, hidden, n, n.cases, h,
				  v, xlab, ylab, xlim, ylim, ...)
			}
		}
		par(err = 0)
	}
	invisible()
}
"summary.rm"<-
function(x, ...)
{
	if(!is.null(cl <- x$call)) {
		cat("Call:\n")
		dput(cl)
	}
	class(x) <- NULL
	cat("\nNumber of profiles used in the fit: ", x$n.cases, "\n"
		)
	coef <- x$coefficients
	nas <- is.na(coef[, 1])
	cnames <- names(coef[, 1][!nas])
	coef <- matrix(rep(coef[, 1][!nas], 4), ncol = 4)
	coef[, 1] <- 1:dim(coef)[[1]]
	coef[, 3] <- x$s.e[, 1][!nas]
	coef[, 4] <- round(coef[, 2]/coef[, 3], 3)
	dimnames(coef) <- list(cnames, c("Label", "Value", 
		"Std. Error", "t value"))
	cat("\nCoefficients:\t\n")
	cr <- diag(1/sqrt(diag(x$covariance)))
	cr <- cr %*% x$covariance %*% cr
	dimnames(cr) <- list(format(coef[, 1]), format(coef[, 1]))
	if(x$family == "gaussian") {
		print(coef)
		cat("\nWithin-subject estimated variance: ", round(x$
			sigma2, 4), "\n")
		if(!is.null(x$rho))
			cat("\nAutoregressive parameter:", round(x$
				rho, 4), "\n")
		if(!is.null(x$delta)) {
			cat("\nRandom effect(s) estimated variance(s):"
				)
			if(is.vector(x$delta))
				print(round(x$delta, 4))
			else {
				cat("\n")
				print(t(round(x$delta, 4)))
			}
		}
		l <- structure(list(call = cl, n.cases = x$n.cases, 
			coefficients = coef, coef.cor = cr, sigma2 = 
			x$sigma2, rho = x$rho, delta = x$delta, 
			log.likelihood = x$log.likelihood, message = 
			x$message), class = "summary.rm")
	}
	else {
		n <- length(cnames)
		print(coef[ - n,  ])
		if(x$family == "binary")
			cat("\nLog odds ratio: \n")
		else cat("\nAutocorrelation coefficient: \n")
		print(round(coef[n,  ], 4))
		l <- structure(list(call = cl, n.cases = x$n.cases, 
			coefficients = coef, coef.cor = cr, 
			log.likelihood = x$log.likelihood, message = 
			x$message), class = "summary.rm")
	}
	cat("\nCorrelation of Coefficients: \n")
	print(cr, digits = 2)
	cat("\nLog likelihood: ", round(x$log.likelihood, 4), "\n")
	cat("\nMessage: ", x$message, "\n")
	invisible(l)
}
"under.unix"<-
F
