*   Subroutines to be used in rm.tools module
*   COPYRIGHT A.Azzalini M.Chiogna 1995
*
*   bin_llik.for   (4/7/95)
*    1) creato modificando loglik_ok.f, per consentire che psi=1 nel
*       calcolo del gradiente
*    2) aggiustata la verifica sul massimo numero di dati mancanti consecutivi


C   likelihood and other functions useful for(beta,psi) parameterization
C   f77 version 
C -----------------------------------------------------------------------
C -----------------------------------------------------------------------
      subroutine bin_llik(logL,beta,log_psi,npar,x,y,theta,work,n)
      implicit double precision (a-h,o-z)
      dimension x(n,npar-1),beta(npar-1),work(n),theta(n),
     +   P0(2,2),P1(2,2),P2(2,2),trans_pr(2),pr(2)
      double precision logL,log_psi
      integer y(n)
      psi = dexp(log_psi)
      psi1 = psi-1
      call matprod(x,beta,work,n,npar-1,1)
      do i=1,n
        theta(i) = 1/(1+dexp(-work(i)))
      end do
      i0 = 1  ! assume NA recoded as -1
      do while(y(i0).eq.(-1)) 
        i0 = i0+1 
      end do
      n0 = n
      do while(y(n0).eq.(-1)) 
        n0 = n0-1
      end do
C     y(i0) is first non-missing datum, y(n0) is the last one
      p  = theta(i0)
      logL = y(i0)*dlog(p/(1-p))+dlog(1-p)
      if(i0.eq.n0) return
      i = i0+1
      do while(i.le.n0)
        i1 = i
        do while(y(i1).eq.(-1)) 
          i1 = i1+1
        end do
        ! now i0 is most recent(past) observation time, 
        ! i1 is next observ.time
        if(i1.eq.i) then   ! no intermediate missing data
          th1 = theta(i)
          th0 = theta(i-1)
          call mc_p(th0,th1,psi,trans_pr)
        else  ! one or more intermediate missing data
          call make_mc(0.0D0,1.0D0,P0)       
          do k=(i0+1),i1
            call mc_p(theta(k-1),theta(k),psi,pr)
            call make_mc(pr(1),pr(2),P1)
            call matprod(P0, P1, P2, 2,2,2)
            call matcopy(P2, P0, 2,2)
          end do
          trans_pr(1)=P0(1,2)
          trans_pr(2)=P0(2,2)
        endif
        p=trans_pr(y(i0)+1)
        logL = logL+  y(i1)*dlog(p/(1-p))+dlog(1-p)
        i0 = i1
        i = i0+1
      end do
      return
      end

C -----------------------------------------------------------------------

      subroutine mc_p(th0,th1,psi,p)
      implicit double precision (a-h,o-z)
      dimension p(2)
      logical unfeasible
      p(1) = th1
      p(2) = th1
      psi1 = psi-1
      if(abs(psi1) .gt. 1.e-10) then 
        delta=dsqrt(1+psi1*((th1-th0)**2*psi-(th0+th1)**2+2*(th0+th1)))
        do j=0,1 
          p(j+1)=((delta-1+psi1*(th1-th0))/(2*psi1*(1-th0))+
     +      j*(1-delta+psi1*(th0+th1-2*th0*th1))/(2*psi1*th0*(1-th0)))
        end do    
      endif
      unfeasible = .FALSE.
      do i=1,2 
        if(p(i) .lt. 0 .or. p(i) .gt.1) unfeasible=.TRUE.
      end do
!      if(unfeasible) stop 'mcp: unfeasible p'
      return
      end

C -----------------------------------------------------------------------

      subroutine make_mc(p1,p2,P)
      ! creates 2x2 MC transition matrix, given probs of 1's
      implicit double precision (a-h,o-z)
      dimension P(2,2)
      P(1,1)=1-p1
      P(1,2)=p1
      P(2,1)=1-p2
      P(2,2)=p2
      return
      end

      subroutine matprod(a,b,c,n1,n2,n3)
      ! computes c=a*b
      implicit double precision (a-h,o-z)
      dimension a(n1,n2),b(n2,n3),c(n1,n3)    
      do i=1,n1
        do j=1,n3
          s=0
            do k=1,n2
              s=s+a(i,k)*b(k,j)
            end do
          c(i,j)=s
        end do
      end do
      return
      end
  
      subroutine matcopy(a,b,n1,n2)
      ! b=a
      implicit double precision (a-h,o-z)
      dimension a(n1,n2),b(n1,n2)
      do i=1,n1
        do j=1,n2
          b(i,j)=a(i,j)
        end do
      end do
      return
      end
!---------------------------------------------------------------------------


!     loglik.odds.gradient(param,dataset,print.level=0)

      subroutine bin_grad(g_beta,g_logpsi,beta,log_psi, 
     +           npar,x,y,theta,work,der,d_beta,d_beta0,d_beta1,n)
      implicit double precision (a-h,o-z)
      dimension x(n,npar-1),beta(npar-1),work(n),theta(n),
     +  P0(2,2),P1(2,2),P2(2,2),trans_pr(2),pr(2),d_theta(3),deriv(3),
     +  der(npar-1),g_beta(npar-1),d_beta0(npar-1),d_beta1(npar-1),
     +  d_beta(3,npar-1)
      double precision log_psi
      integer y(n),k1,k2,i0,i1
      psi = dexp(log_psi)
      psi1 = psi-1
      call matprod(x,beta,work,n,npar-1,1)
      do i=1,n
        theta(i) = 1/(1+dexp(-work(i)))
      end do
      i0 = 1  ! assume NA recoded as -1
      do while(y(i0).eq.(-1)) 
        i0 = i0+1 
      end do
      n0 = n
      do while(y(n0).eq.(-1)) 
        n0 = n0-1
      end do
      ! y(i0) is first non-missing datum, y(n0) is the last one
      p  = theta(i0)
      do k2=1,npar-1
        g_beta(k2)=(y(i0)/(theta(i0)*(1-theta(i0)))-1/(1-theta(i0))) * 
     *                      theta(i0)*(1-theta(i0))*x(i0,k2)
        d_beta0(k2) =  theta(i0)*(1-theta(i0))*x(i0,k2)
      end do     
      g_logpsi = 0
      if(i0.eq.n0) return
      g_psi = 0
      i = i0+1
      do while(i.le.n0)
        i1 = i
        do while(y(i1).eq.(-1)) 
          i1 = i1+1
        end do
        !  now i0 is most recent(past) observation time, 
        !  i1 is next observ.time
        if(i1.eq.(i0+1)) then
          ! no intermediate missing data(i1=i0+1)
          j  = y(i0)
          call mc_p(theta(i-1),theta(i),psi,trans_pr)
          call deriv1(theta,psi,i,j,deriv)
          d_theta0 = deriv(1)
          d_theta1 = deriv(2)
          d_p_psi = deriv(3)
          p = trans_pr(j+1)
          d_p = y(i1)/(p*(1-p))-1/(1-p) 
          do k2=1,npar-1
            d_beta1(k2) = theta(i)*(1-theta(i))*x(i,k2)
            d_beta0(k2) = theta(i-1)*(1-theta(i-1))*x(i-1,k2)
            g_beta(k2) = g_beta(k2) +
     +          d_p*(d_theta1*d_beta1(k2)+d_theta0*d_beta0(k2))
          end do
          g_psi = g_psi+d_p*d_p_psi
        else
          !   Case with (exactly) one intermediate missing datum (i1=i0+2)
          !   In effetti, passa per di qua' anche se i1>i0+2.  Cio' non e'
          !   proprio corretto (ma l'inaccuratezza puo' forse errore
          !   minima). Assumiamo che il programma chiamante abbia gia'
          !   fatto una verifica al proposito, ed emesso un avvertimento.
          call mc_p(theta(i0),theta(i0+1),psi,pr)
          call make_mc(pr(1),pr(2),P1)
          call mc_p(theta(i0+1),theta(i1),psi,pr)
          call make_mc(pr(1),pr(2),P2)
          call matprod(P1,P2,P0,2,2,2)
          j = y(i0)
          trans_pr(1) = P0(1,2)
          trans_pr(2) = P0(2,2)
          p = trans_pr(j+1)
          d_p = y(i1)/(p*(1-p))-1/(1-p)  
          ! d_beta = theta(i0:i1)*(1-theta(i0:i1))*x(i0:i1,)
          do k1=0,2
            do k2=1,npar-1
              d_beta(k1+1,k2)=theta(k1+i0)*(1-theta(k1+i0))*x(k1+i0,k2)
            end do
          end do        
          call deriv1(theta,psi,i0+1,j,deriv)
          d_theta(1) = deriv(1)*(-P2(1,2)+P2(2,2)) ! this is done
          d_theta(2) = deriv(2)*(-P2(1,2)+P2(2,2)) ! not yet complete
          d_p_psi = deriv(3)*(-P2(1,2)+P2(2,2))    ! not yet complete
          call deriv1(theta,psi,i1,0,deriv)
          d_theta(2) = d_theta(2)+(1-P1(j+1,2))*deriv(1)
          d_theta(3) =(1-P1(j+1,2))*deriv(2)
          d_p_psi = d_p_psi +(1-P1(j+1,2))*deriv(3)
          call deriv1(theta,psi,i1,1,deriv)
          d_theta(2) = d_theta(2)+deriv(1)*P1(j+1,2)
          d_theta(3) = d_theta(3)+deriv(2)*P1(j+1,2)
          d_p_psi = d_p_psi + P1(j+1,2)*deriv(3)
          ! deriv <- as.matrix(d_theta*d_beta,3,n.par-1)
          ! der <- apply(deriv,2,sum)
          do k2=1,npar-1
            der(k2)=0.0d0
            do k1=1,3
              der(k2)=der(k2)+d_theta(k1)*d_beta(k1,k2)
            end do
          end do    
          do k2=1,npar-1    
            g_beta(k2) = g_beta(k2) + d_p*der(k2)
          end do
          g_psi = g_psi + d_p*d_p_psi
        endif
        i0 = i1
        i = i0+1
      end do ! (end while)
      g_logpsi=g_psi*psi
      return
      end

C -----------------------------------------------------------------------
C devivatives of p(t-1,t|j), i.e. prob{Y(t)=1|y(t-1)=j}
C wr to theta(t-1), theta(t), psi

      subroutine deriv1(theta,psi,t,j,der)
      implicit double precision (a-h,o-z)
      integer t,j,j1
      dimension theta(t),der(3)
      th0 = theta(t-1)
      th1 = theta(t)
      psi1 = psi-1
      if(abs(psi1) .gt. 1.0e-6) then
        j1 = 2*j-1
        delta=dsqrt(1+psi1*((th1-th0)**2*psi-(th0+th1)**2+2*(th0+th1)))
        d1=psi1*(psi*(th1-th0)-(th1+th0)+1)/delta
        d0=psi1*(-psi*(th1-th0)-(th1+th0)+1)/delta
        dpsi=((th1-th0)**2*(2*psi-1)-(th1+th0)**2+2*(th1+th0))/(2*delta)
        A=(2*psi1*(1-j+j1*th0))
        B=(1-delta+psi1*th0)*j1+th1*psi1
        d_theta1 =(-j1*d1+psi1)/A
        d_theta0 =(j1*(psi1-d0)*A-2*psi1*j1*B)/A**2
        d_p_psi  =((j1*(-dpsi+th0)+th1)*A-2*B*(1-j+j1*th0))/A**2
        der(1)=d_theta0
        der(2)=d_theta1
        der(3)=d_p_psi
      else
        der(1)=0
        der(2)=1
        der(3)=(th0-j)*(th1**2-th1)
      end if
      return
      end  




C -----------------------------------------------------------------------

*  PoiSSon_LogLIKelihood: computes the log-likelihood for a
*  Poisson model with covariates x(,) and serial correlation rho
*    link = 0: identity
*    link = 1: log

      subroutine pss_llik(logL,beta,rho,np,x,y,theta,work,n,fact,link)
      implicit double precision (a-h,o-z)
      dimension x(n,np-1),beta(np-1),work(n),theta(n),fact(0:1)
      double precision logL
      integer y(n)
      data zero/0.0d0/
      call matprod(x,beta,work,n,np-1,1)  ! work=X'beta
      i0=0
      do i=1,n
        if(link.eq.0) then
          theta(i)=work(i)          ! link identity
          if(theta(i).le.zero) then
            call intpr('Unfeasible value, component',27,i,1)
            call dblepr('Poisson mean value is',21,theta(i),1)
          endif
        elseif(link.eq.1) then
          theta(i) = dexp(work(i))  ! link log
        else
          call intpr('Poisson model with invalid link',31,link,1)
        endif
      end do
      i0 = 1  ! assume NA recoded as -1
      do while(y(i0).eq.(-1)) 
        i0 = i0+1 
      end do
      n0 = n
      do while(y(n0).eq.(-1)) 
        n0 = n0-1
      end do
C     y(i0) is first non-missing datum, y(n0) is the last one

*  il primo termine di logL e' calcolato con distrib.Poisson
      logL = -theta(i0)+ y(i0)*dlog(theta(i0)) 
      if(i0.eq.n0) return
      i = i0+1
*  ciclo per i restanti termini di logL
      do while(i.le.n0)
        i1 = i
        do while(y(i1).eq.(-1)) 
          i1 = i1+1
        end do
        ! now i0 is most recent(past) observation time, 
        ! i1 is next observ.time
        prob=tp_poisson(i0,y(i0),i1,y(i1),theta,rho,fact)
        logL = logL+ dlog(prob*fact(y(i1)))   
        ! moltiplico prob per fact(y(i1)) per prevenire -Inf,
        ! inoltre questo sostanzialmente e` simile a togliere il fact() 
        ! a denominatore in una Poisson
        i0 = i1
        i = i0+1
      end do
      return
      end

C -----------------------------------------------------------------------
*  tp_poisson : transition probability for Poisson model

*  Calcolo probabilita' di transizione di un  processo con distribuzione
*  marginale di Poisson, secondo la formula (8) dell'articolo JISS 1995.
*  L'evento di cui si calcola la prob. e' la transizione a partire dallo
*  stato k0 occupato al tempo t0, verso lo stato k1 at tempo t1.

      double precision function tp_poisson(t0,k0,t1,k1,theta,rho,f)
      implicit double precision (a-h,o-z)
      integer t1,t0,r
      double precision nu
      dimension theta(t1),f(0:20)
      ! si assume che f() contenga la sequenza dei fattoriali
      nu=theta(t1)-rho*theta(t0)
      if(nu.le.0.0) then
        call dblepr('Parameters values unfeasible',28,nu,1)
        return
        endif
      maxr=min(k1,k0)
      prob=0.0d0
      rhom=rho**(t1-t0)
      do r=0,maxr
        a=f(k0)* rhom**r *(1-rhom)**(k0-r) *exp(-nu) * nu**(k1-r)
        b=f(r)*f(k0-r)*f(k1-r)
        prob=prob+a/b
      end do
      tp_poisson=prob
      return
      end

C -----------------------------------------------------------------------

!  Gradiente  della log-verosimiglianza di Poisson, output in grad.
!  Le prime npar-1 componenti sono relative a beta, l'ultima riguarda rho

      subroutine pss_grad(grad,beta,rho,npar,x,y,theta,work,n,f,link)
      implicit double precision (a-h,o-z)
      dimension grad(npar),x(n,npar-1),beta(npar-1),work(n),
     +          theta(n),f(0:1)
      double precision nu
      integer y(n),p,r,rmax
      G(r,k) = rm**r *(1-rm)**(k-r)
      H(r) = expnu * nu**r /f(r)
      data zero/0.0d0/, one/1.0d0/
      p=npar-1
      call matprod(x,beta,work,n,p,1)  ! work=X'beta
      do i=1,n
        if(link.eq.0) then
          theta(i)= work(i)
          if(theta(i).le.zero) then
            call intpr('Unfeasible value, component',27,i,1)
            call dblepr('Poisson mean value is',21,theta(i),1)
          endif
        elseif(link.eq.1) then
          theta(i) = dexp(work(i))  ! link log
        else
          call intpr('Poisson model with invalid link',31,link,1)
        endif
      end do
      i0 = 1  ! assume NA recoded as -1
      do while(y(i0).eq.(-1)) 
        i0 = i0+1 
      end do
      n0 = n
      do while(y(n0).eq.(-1)) 
        n0 = n0-1
      end do
C     y(i0) is first non-missing datum, y(n0) is the last one

*  calcoli relativi al primo valore osservato
      do k=1,p
        if(link.eq.0) then
          gl=one
        elseif(link.eq.1) then
          gl=theta(i0)
        else
          gl=zero
        endif
        grad(k)=(-one+y(i0)/theta(i0)) * gl * x(i0,k)
      end do
      grad(npar)=zero
      if(i0.eq.n0) return
      i = i0+1
*  ciclo per i restanti termini del gradiente
      do while(i.le.n0)
        i1 = i
        do while(y(i1).eq.(-1)) 
          i1 = i1+1
        end do
        ! now i0 is most recent(past) observation time, 
        ! i1 is next observ.time
        !
        ! calcolo grad per le componenti relative a beta
        rm=rho**(i1-i0)
        k1=y(i1)
        k0=y(i0)
        prob=tp_poisson(i0,k0,i1,k1,theta,rho,f)
        a=-prob
        if(k1.gt.0) a=a+tp_poisson(i0,k0,i1,k1-1,theta,rho,f)
        do ip=1,p
          if(link.eq.0) then
            gl0=one
            gl1=one
          elseif(link.eq.1) then
            gl0=theta(i0)
            gl1=theta(i1)
          else
            gl=zero
          endif
          b=gl1*x(i1,ip)-rm*gl0*x(i0,ip)
          grad(ip)=grad(ip)+a*b/prob
        end do
        ! calcolo grad per la componente relativa a rho
        rmax=min(k1,k0)
        nu=theta(i1)-rm*theta(i0)
        expnu=dexp(-nu)
        dp_rm=zero
        do r=0,rmax
          comb=f(k0)/(f(r)*f(k0-r))
          dg=r*G(r-1,k0-1)-(k0-r)*G(r,k0-1)
          dh=-H(k1-r)
          if(k1.gt.r) dh=dh+H(k1-r-1)
          dh=dh*(-theta(i0))
          dp_rm=dp_rm+comb*(dg*H(k1-r)+G(r,k0)*dh)
        end do
        grad(npar)=grad(npar)+(i1-i0)*rm*dp_rm/(prob*rho)
        ! finito calcolo grad per rho
        i0 = i1
        i = i0+1
      end do
      return
      end
